/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_JSONS_H_
#define _IPWORKSSSL_JSONS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthSchemes
#define AUTH_BASIC                                         0
#define AUTH_DIGEST                                        1
#define AUTH_PROPRIETARY                                   2
#define AUTH_NONE                                          3
#define AUTH_NTLM                                          4
#define AUTH_NEGOTIATE                                     5
#define AUTH_OAUTH                                         6

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TFollowRedirects
#define FR_NEVER                                           0
#define FR_ALWAYS                                          1
#define FR_SAME_SCHEME                                     2

//ProxySSLTypes
#define PS_AUTOMATIC                                       0
#define PS_ALWAYS                                          1
#define PS_NEVER                                           2
#define PS_TUNNEL                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_JSONS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_JSONS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_JSONS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_JSONS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_JSONS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_JSONS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_JSONS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_JSONS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_JSONS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_JSONS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  const char* Text;
  int lenText;
  int reserved;
} JSONSCharactersEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} JSONSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} JSONSConnectionStatusEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} JSONSDisconnectedEventParams;

typedef struct {
  const char* Element;
  int reserved;
} JSONSEndElementEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} JSONSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} JSONSHeaderEventParams;

typedef struct {
  const char* Text;
  int reserved;
} JSONSIgnorableWhitespaceEventParams;

typedef struct {
  const char* Location;
  int Accept;
  int reserved;
} JSONSRedirectEventParams;

typedef struct {
  const char* Name;
  const char* Value;
  const char* Expires;
  const char* Domain;
  const char* Path;
  int Secure;
  int reserved;
} JSONSSetCookieEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} JSONSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} JSONSSSLStatusEventParams;

typedef struct {
  const char* Element;
  int reserved;
} JSONSStartElementEventParams;

typedef struct {
  const char* HTTPVersion;
  int StatusCode;
  const char* Description;
  int reserved;
} JSONSStatusEventParams;



class JSONS {
  
  public: //events
  
    virtual int FireCharacters(JSONSCharactersEventParams *e) {return 0;}
    virtual int FireConnected(JSONSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(JSONSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDisconnected(JSONSDisconnectedEventParams *e) {return 0;}
    virtual int FireEndElement(JSONSEndElementEventParams *e) {return 0;}
    virtual int FireError(JSONSErrorEventParams *e) {return 0;}
    virtual int FireHeader(JSONSHeaderEventParams *e) {return 0;}
    virtual int FireIgnorableWhitespace(JSONSIgnorableWhitespaceEventParams *e) {return 0;}
    virtual int FireRedirect(JSONSRedirectEventParams *e) {return 0;}
    virtual int FireSetCookie(JSONSSetCookieEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(JSONSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(JSONSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartElement(JSONSStartElementEventParams *e) {return 0;}
    virtual int FireStatus(JSONSStatusEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL JSONSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((JSONS*)lpObj)->JSONSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            JSONSCharactersEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((JSONS*)lpObj)->FireCharacters(&e);
            break;
         }
         case 2: {
            JSONSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((JSONS*)lpObj)->FireConnected(&e);
            break;
         }
         case 3: {
            JSONSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((JSONS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 4: {
            JSONSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((JSONS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 5: {
            JSONSEndElementEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((JSONS*)lpObj)->FireEndElement(&e);
            break;
         }
         case 6: {
            JSONSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((JSONS*)lpObj)->FireError(&e);
            break;
         }
         case 7: {
            JSONSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((JSONS*)lpObj)->FireHeader(&e);
            break;
         }
         case 8: {
            JSONSIgnorableWhitespaceEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((JSONS*)lpObj)->FireIgnorableWhitespace(&e);
            break;
         }
         case 9: {
            JSONSRedirectEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = ((JSONS*)lpObj)->FireRedirect(&e);
            param[1] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 10: {
            JSONSSetCookieEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = ((JSONS*)lpObj)->FireSetCookie(&e);
            break;
         }
         case 11: {
            JSONSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((JSONS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 12: {
            JSONSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((JSONS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 13: {
            JSONSStartElementEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((JSONS*)lpObj)->FireStartElement(&e);
            break;
         }
         case 14: {
            JSONSStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((JSONS*)lpObj)->FireStatus(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int JSONSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    JSONS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_68) {
      m_pObj = IPWorksSSL_JSONS_Create(JSONSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~JSONS() {
      IPWorksSSL_JSONS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_JSONS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_JSONS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_JSONS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAccept() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAccept(const char *lpAccept) {
      return IPWorksSSL_JSONS_Set(m_pObj, 1, 0, (void*)lpAccept, 0);
    }

    inline char* GetAuthorization() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }

    inline int SetAuthorization(const char *lpAuthorization) {
      return IPWorksSSL_JSONS_Set(m_pObj, 2, 0, (void*)lpAuthorization, 0);
    }

    inline int GetAuthScheme() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthScheme(int iAuthScheme) {
      void* val = (void*)IPS64CAST(iAuthScheme);
      return IPWorksSSL_JSONS_Set(m_pObj, 3, 0, val, 0);
    }
    inline int GetBuildDOM() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetBuildDOM(int bBuildDOM) {
      void* val = (void*)IPS64CAST(bBuildDOM);
      return IPWorksSSL_JSONS_Set(m_pObj, 4, 0, val, 0);
    }
    inline int GetConnected() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 5, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_JSONS_Set(m_pObj, 5, 0, val, 0);
    }
    inline char* GetContentType() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 6, 0, 0);
      return (char*)val;
    }

    inline int SetContentType(const char *lpContentType) {
      return IPWorksSSL_JSONS_Set(m_pObj, 6, 0, (void*)lpContentType, 0);
    }

    inline int GetCookieCount() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 7, 0, 0);
      return (int)(long)val;
    }
    inline int SetCookieCount(int iCookieCount) {
      void* val = (void*)IPS64CAST(iCookieCount);
      return IPWorksSSL_JSONS_Set(m_pObj, 7, 0, val, 0);
    }
    inline char* GetCookieDomain(int iCookieIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 8, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieExpiration(int iCookieIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 9, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieName(int iCookieIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 10, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieName(int iCookieIndex, const char *lpCookieName) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline char* GetCookiePath(int iCookieIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 11, iCookieIndex, 0);
      return (char*)val;
    }


    inline int GetCookieSecure(int iCookieIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 12, iCookieIndex, 0);
      return (int)(long)val;
    }

    inline char* GetCookieValue(int iCookieIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 13, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieValue(int iCookieIndex, const char *lpCookieValue) {
      return IPWorksSSL_JSONS_Set(m_pObj, 13, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 14, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_JSONS_Set(m_pObj, 14, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 15, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_JSONS_Set(m_pObj, 15, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_JSONS_Set(m_pObj, 16, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 17, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 18, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_JSONS_Set(m_pObj, 18, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_JSONS_Set(m_pObj, 19, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetFollowRedirects() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 20, 0, 0);
      return (int)(long)val;
    }
    inline int SetFollowRedirects(int iFollowRedirects) {
      void* val = (void*)IPS64CAST(iFollowRedirects);
      return IPWorksSSL_JSONS_Set(m_pObj, 20, 0, val, 0);
    }
    inline char* GetFrom() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetFrom(const char *lpFrom) {
      return IPWorksSSL_JSONS_Set(m_pObj, 21, 0, (void*)lpFrom, 0);
    }

    inline char* GetHTTPMethod() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetHTTPMethod(const char *lpHTTPMethod) {
      return IPWorksSSL_JSONS_Set(m_pObj, 22, 0, (void*)lpHTTPMethod, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }

    inline char* GetIfModifiedSince() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }

    inline int SetIfModifiedSince(const char *lpIfModifiedSince) {
      return IPWorksSSL_JSONS_Set(m_pObj, 24, 0, (void*)lpIfModifiedSince, 0);
    }

    inline char* GetLocalFile() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_JSONS_Set(m_pObj, 25, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_JSONS_Set(m_pObj, 26, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 27, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_JSONS_Set(m_pObj, 27, 0, (void*)lpOtherHeaders, 0);
    }

    inline int GetParsedHeaderCount() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 28, 0, 0);
      return (int)(long)val;
    }

    inline char* GetParsedHeaderField(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 29, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetParsedHeaderValue(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 30, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 31, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 31, 0, (void*)lpPassword, 0);
    }

    inline int GetPostData(char *&lpPostData, int &lenPostData) {
      lpPostData = (char*)IPWorksSSL_JSONS_Get(m_pObj, 32, 0, &lenPostData);
      return lpPostData ? 0 : lenPostData;
    }

    inline int SetPostData(const char *lpPostData, int lenPostData) {
      return IPWorksSSL_JSONS_Set(m_pObj, 32, 0, (void*)lpPostData, lenPostData);
    }

    inline int GetProxyAuthScheme() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 33, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAuthScheme(int iProxyAuthScheme) {
      void* val = (void*)IPS64CAST(iProxyAuthScheme);
      return IPWorksSSL_JSONS_Set(m_pObj, 33, 0, val, 0);
    }
    inline int GetProxyAutoDetect() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 34, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAutoDetect(int bProxyAutoDetect) {
      void* val = (void*)IPS64CAST(bProxyAutoDetect);
      return IPWorksSSL_JSONS_Set(m_pObj, 34, 0, val, 0);
    }
    inline char* GetProxyPassword() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 35, 0, 0);
      return (char*)val;
    }

    inline int SetProxyPassword(const char *lpProxyPassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 35, 0, (void*)lpProxyPassword, 0);
    }

    inline int GetProxyPort() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 36, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyPort(int lProxyPort) {
      void* val = (void*)IPS64CAST(lProxyPort);
      return IPWorksSSL_JSONS_Set(m_pObj, 36, 0, val, 0);
    }
    inline char* GetProxyServer() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 37, 0, 0);
      return (char*)val;
    }

    inline int SetProxyServer(const char *lpProxyServer) {
      return IPWorksSSL_JSONS_Set(m_pObj, 37, 0, (void*)lpProxyServer, 0);
    }

    inline int GetProxySSL() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 38, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxySSL(int iProxySSL) {
      void* val = (void*)IPS64CAST(iProxySSL);
      return IPWorksSSL_JSONS_Set(m_pObj, 38, 0, val, 0);
    }
    inline char* GetProxyUser() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 39, 0, 0);
      return (char*)val;
    }

    inline int SetProxyUser(const char *lpProxyUser) {
      return IPWorksSSL_JSONS_Set(m_pObj, 39, 0, (void*)lpProxyUser, 0);
    }

    inline char* GetReferer() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 40, 0, 0);
      return (char*)val;
    }

    inline int SetReferer(const char *lpReferer) {
      return IPWorksSSL_JSONS_Set(m_pObj, 40, 0, (void*)lpReferer, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_JSONS_Get(m_pObj, 41, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_JSONS_Set(m_pObj, 41, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_JSONS_Get(m_pObj, 42, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_JSONS_Set(m_pObj, 42, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_JSONS_Get(m_pObj, 43, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_JSONS_Set(m_pObj, 43, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 44, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 44, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 45, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_JSONS_Set(m_pObj, 45, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 46, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_JSONS_Set(m_pObj, 46, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_JSONS_Get(m_pObj, 47, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline char* GetStatusLine() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 48, 0, 0);
      return (char*)val;
    }


    inline int GetTimeout() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 49, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_JSONS_Set(m_pObj, 49, 0, val, 0);
    }
    inline int GetTransferredData(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_JSONS_Get(m_pObj, 50, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }

    inline int SetTransferredData(const char *lpTransferredData, int lenTransferredData) {
      return IPWorksSSL_JSONS_Set(m_pObj, 50, 0, (void*)lpTransferredData, lenTransferredData);
    }

    inline ns_int64 GetTransferredDataLimit() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_JSONS_Get(m_pObj, 51, 0, 0);
      return *pval;
    }

    inline int SetTransferredDataLimit(ns_int64 lTransferredDataLimit) {
      void* val = (void*)(&lTransferredDataLimit);
      return IPWorksSSL_JSONS_Set(m_pObj, 51, 0, val, 0);
    }

    inline char* GetTransferredHeaders() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 52, 0, 0);
      return (char*)val;
    }


    inline char* GetURL() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 53, 0, 0);
      return (char*)val;
    }

    inline int SetURL(const char *lpURL) {
      return IPWorksSSL_JSONS_Set(m_pObj, 53, 0, (void*)lpURL, 0);
    }

    inline char* GetUser() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 54, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_JSONS_Set(m_pObj, 54, 0, (void*)lpUser, 0);
    }

    inline int GetXChildrenCount() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 55, 0, 0);
      return (int)(long)val;
    }

    inline char* GetXChildName(int iXChildIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 56, iXChildIndex, 0);
      return (char*)val;
    }


    inline char* GetXChildXText(int iXChildIndex) {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 57, iXChildIndex, 0);
      return (char*)val;
    }


    inline char* GetXElement() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 58, 0, 0);
      return (char*)val;
    }


    inline char* GetXErrorPath() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 59, 0, 0);
      return (char*)val;
    }

    inline int SetXErrorPath(const char *lpXErrorPath) {
      return IPWorksSSL_JSONS_Set(m_pObj, 59, 0, (void*)lpXErrorPath, 0);
    }

    inline char* GetXParent() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 60, 0, 0);
      return (char*)val;
    }


    inline char* GetXPath() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 61, 0, 0);
      return (char*)val;
    }

    inline int SetXPath(const char *lpXPath) {
      return IPWorksSSL_JSONS_Set(m_pObj, 61, 0, (void*)lpXPath, 0);
    }

    inline char* GetXSubTree() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 62, 0, 0);
      return (char*)val;
    }


    inline char* GetXText() {
      void* val = IPWorksSSL_JSONS_Get(m_pObj, 63, 0, 0);
      return (char*)val;
    }



  public: //methods

    inline int AddCookie(const char* lpszCookieName, const char* lpszCookieValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszCookieName), (void*)IPS64CAST(lpszCookieValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 2, 2, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_JSONS_Do(m_pObj, 3, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Delete(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 4, 1, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_JSONS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline int Get(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 6, 1, param, cbparam);
      
      
    }
    inline int HasXPath(const char* lpszxpath) {
      void *param[1+1] = {(void*)IPS64CAST(lpszxpath), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_JSONS_Do(m_pObj, 7, 1, param, cbparam);
      
      return (int)IPS64CAST(param[1]);
    }
    inline int Input(const char* lpszText) {
      void *param[1+1] = {(void*)IPS64CAST(lpszText), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 8, 1, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_JSONS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int ParseFile(const char* lpszFileName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszFileName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10, 1, param, cbparam);
      
      
    }
    inline int Post(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 11, 1, param, cbparam);
      
      
    }
    inline int Put(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 12, 1, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_JSONS_Do(m_pObj, 13, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  LPWSTR Text;
  int lenText;
  int reserved;
} JSONSCharactersEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} JSONSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} JSONSConnectionStatusEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} JSONSDisconnectedEventParamsW;

typedef struct {
  LPWSTR Element;
  int reserved;
} JSONSEndElementEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} JSONSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} JSONSHeaderEventParamsW;

typedef struct {
  LPWSTR Text;
  int reserved;
} JSONSIgnorableWhitespaceEventParamsW;

typedef struct {
  LPWSTR Location;
  int Accept;
  int reserved;
} JSONSRedirectEventParamsW;

typedef struct {
  LPWSTR Name;
  LPWSTR Value;
  LPWSTR Expires;
  LPWSTR Domain;
  LPWSTR Path;
  int Secure;
  int reserved;
} JSONSSetCookieEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} JSONSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} JSONSSSLStatusEventParamsW;

typedef struct {
  LPWSTR Element;
  int reserved;
} JSONSStartElementEventParamsW;

typedef struct {
  LPWSTR HTTPVersion;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} JSONSStatusEventParamsW;



class JSONSW : public JSONS {

  public: //properties
  
    inline LPWSTR GetAccept() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAccept(LPWSTR lpAccept) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+1, 0, (void*)lpAccept, 0);
    }

    inline LPWSTR GetAuthorization() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+2, 0, 0);
    }

    inline int SetAuthorization(LPWSTR lpAuthorization) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+2, 0, (void*)lpAuthorization, 0);
    }







    inline LPWSTR GetContentType() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+6, 0, 0);
    }

    inline int SetContentType(LPWSTR lpContentType) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+6, 0, (void*)lpContentType, 0);
    }



    inline LPWSTR GetCookieDomain(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+8, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieExpiration(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+9, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieName(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+10, iCookieIndex, 0);
    }

    inline int SetCookieName(int iCookieIndex, LPWSTR lpCookieName) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+10, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline LPWSTR GetCookiePath(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+11, iCookieIndex, 0);
    }





    inline LPWSTR GetCookieValue(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+13, iCookieIndex, 0);
    }

    inline int SetCookieValue(int iCookieIndex, LPWSTR lpCookieValue) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+13, iCookieIndex, (void*)lpCookieValue, 0);
    }





    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+16, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+17, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+19, 0, (void*)lpFirewallUser, 0);
    }



    inline LPWSTR GetFrom() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetFrom(LPWSTR lpFrom) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+21, 0, (void*)lpFrom, 0);
    }

    inline LPWSTR GetHTTPMethod() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetHTTPMethod(LPWSTR lpHTTPMethod) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+22, 0, (void*)lpHTTPMethod, 0);
    }



    inline LPWSTR GetIfModifiedSince() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetIfModifiedSince(LPWSTR lpIfModifiedSince) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+24, 0, (void*)lpIfModifiedSince, 0);
    }

    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+10000+25, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+10000+25, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+26, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+26, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+27, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+27, 0, (void*)lpOtherHeaders, 0);
    }



    inline LPWSTR GetParsedHeaderField(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+29, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetParsedHeaderValue(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+30, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+31, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+31, 0, (void*)lpPassword, 0);
    }

    inline LPWSTR GetPostData() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+32, 0, 0);
    }

    inline int SetPostData(LPWSTR lpPostData) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+32, 0, (void*)lpPostData, 0);
    }
    inline int GetPostDataB(char *&lpPostData, int &lenPostData) {
      lpPostData = (char*)IPWorksSSL_JSONS_Get(m_pObj, 32, 0, &lenPostData);
      return lpPostData ? 0 : lenPostData;
    }
    inline int SetPostDataB(const char *lpPostData, int lenPostData) {
      return IPWorksSSL_JSONS_Set(m_pObj, 32, 0, (void*)lpPostData, lenPostData);
    }




    inline LPWSTR GetProxyPassword() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+35, 0, 0);
    }

    inline int SetProxyPassword(LPWSTR lpProxyPassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+35, 0, (void*)lpProxyPassword, 0);
    }



    inline LPWSTR GetProxyServer() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+37, 0, 0);
    }

    inline int SetProxyServer(LPWSTR lpProxyServer) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+37, 0, (void*)lpProxyServer, 0);
    }



    inline LPWSTR GetProxyUser() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+39, 0, 0);
    }

    inline int SetProxyUser(LPWSTR lpProxyUser) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+39, 0, (void*)lpProxyUser, 0);
    }

    inline LPWSTR GetReferer() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+40, 0, 0);
    }

    inline int SetReferer(LPWSTR lpReferer) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+40, 0, (void*)lpReferer, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+41, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+41, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_JSONS_Get(m_pObj, 41, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_JSONS_Set(m_pObj, 41, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+42, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+42, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_JSONS_Get(m_pObj, 42, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_JSONS_Set(m_pObj, 42, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+43, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+43, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_JSONS_Get(m_pObj, 43, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_JSONS_Set(m_pObj, 43, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+44, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+44, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+46, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+46, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+47, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_JSONS_Get(m_pObj, 47, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }

    inline LPWSTR GetStatusLine() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+48, 0, 0);
    }





    inline LPWSTR GetTransferredData() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+50, 0, 0);
    }

    inline int SetTransferredData(LPWSTR lpTransferredData) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+50, 0, (void*)lpTransferredData, 0);
    }
    inline int GetTransferredDataB(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_JSONS_Get(m_pObj, 50, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }
    inline int SetTransferredDataB(const char *lpTransferredData, int lenTransferredData) {
      return IPWorksSSL_JSONS_Set(m_pObj, 50, 0, (void*)lpTransferredData, lenTransferredData);
    }


    inline LPWSTR GetTransferredHeaders() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+52, 0, 0);
    }



    inline LPWSTR GetURL() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+53, 0, 0);
    }

    inline int SetURL(LPWSTR lpURL) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+53, 0, (void*)lpURL, 0);
    }

    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+54, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+54, 0, (void*)lpUser, 0);
    }



    inline LPWSTR GetXChildName(int iXChildIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+56, iXChildIndex, 0);
    }



    inline LPWSTR GetXChildXText(int iXChildIndex) {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+57, iXChildIndex, 0);
    }



    inline LPWSTR GetXElement() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+58, 0, 0);
    }



    inline LPWSTR GetXErrorPath() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+59, 0, 0);
    }

    inline int SetXErrorPath(LPWSTR lpXErrorPath) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+59, 0, (void*)lpXErrorPath, 0);
    }

    inline LPWSTR GetXParent() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+60, 0, 0);
    }



    inline LPWSTR GetXPath() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+61, 0, 0);
    }

    inline int SetXPath(LPWSTR lpXPath) {
      return IPWorksSSL_JSONS_Set(m_pObj, 10000+61, 0, (void*)lpXPath, 0);
    }

    inline LPWSTR GetXSubTree() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+62, 0, 0);
    }



    inline LPWSTR GetXText() {
      return (LPWSTR)IPWorksSSL_JSONS_Get(m_pObj, 10000+63, 0, 0);
    }





  public: //events
  
    virtual int FireCharacters(JSONSCharactersEventParamsW *e) {return 0;}
    virtual int FireConnected(JSONSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(JSONSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDisconnected(JSONSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireEndElement(JSONSEndElementEventParamsW *e) {return 0;}
    virtual int FireError(JSONSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(JSONSHeaderEventParamsW *e) {return 0;}
    virtual int FireIgnorableWhitespace(JSONSIgnorableWhitespaceEventParamsW *e) {return 0;}
    virtual int FireRedirect(JSONSRedirectEventParamsW *e) {return 0;}
    virtual int FireSetCookie(JSONSSetCookieEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(JSONSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(JSONSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartElement(JSONSStartElementEventParamsW *e) {return 0;}
    virtual int FireStatus(JSONSStatusEventParamsW *e) {return 0;}


  protected:
  
    virtual int JSONSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            JSONSCharactersEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireCharacters(&e);
            break;
         }
         case 2: {
            JSONSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 3: {
            JSONSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 4: {
            JSONSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 5: {
            JSONSEndElementEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireEndElement(&e);
            break;
         }
         case 6: {
            JSONSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 7: {
            JSONSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 8: {
            JSONSIgnorableWhitespaceEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireIgnorableWhitespace(&e);
            break;
         }
         case 9: {
            JSONSRedirectEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = FireRedirect(&e);
            param[1] = (void*)(e.Accept);
            break;
         }
         case 10: {
            JSONSSetCookieEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = FireSetCookie(&e);
            break;
         }
         case 11: {
            JSONSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 12: {
            JSONSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 13: {
            JSONSStartElementEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireStartElement(&e);
            break;
         }
         case 14: {
            JSONSStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireStatus(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireCharacters(JSONSCharactersEventParams *e) {return -10000;}
    virtual int FireConnected(JSONSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(JSONSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDisconnected(JSONSDisconnectedEventParams *e) {return -10000;}
    virtual int FireEndElement(JSONSEndElementEventParams *e) {return -10000;}
    virtual int FireError(JSONSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(JSONSHeaderEventParams *e) {return -10000;}
    virtual int FireIgnorableWhitespace(JSONSIgnorableWhitespaceEventParams *e) {return -10000;}
    virtual int FireRedirect(JSONSRedirectEventParams *e) {return -10000;}
    virtual int FireSetCookie(JSONSSetCookieEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(JSONSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(JSONSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartElement(JSONSStartElementEventParams *e) {return -10000;}
    virtual int FireStatus(JSONSStatusEventParams *e) {return -10000;}

  public: //methods

    inline int AddCookie(LPWSTR lpszCookieName, LPWSTR lpszCookieValue) {
      void *param[2+1] = {(void*)lpszCookieName, (void*)lpszCookieValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+2, 2, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_JSONS_Do(m_pObj, 10000+3, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Delete(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+4, 1, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline int Get(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+6, 1, param, cbparam);
      
    }
    inline int HasXPath(LPWSTR lpszxpath) {
      void *param[1+1] = {(void*)lpszxpath, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_JSONS_Do(m_pObj, 10000+7, 1, param, cbparam);
      return (int)IPS64CAST(param[1]);
    }
    inline int Input(LPWSTR lpszText) {
      void *param[1+1] = {(void*)lpszText, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+8, 1, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int ParseFile(LPWSTR lpszFileName) {
      void *param[1+1] = {(void*)lpszFileName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+10, 1, param, cbparam);
      
    }
    inline int Post(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+11, 1, param, cbparam);
      
    }
    inline int Put(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+12, 1, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_JSONS_Do(m_pObj, 10000+13, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_JSONS_H_




