/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_NNTPS_H_
#define _IPWORKSSSL_NNTPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_NNTPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_NNTPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_NNTPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_NNTPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_NNTPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_NNTPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_NNTPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_NNTPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_NNTPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_NNTPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} NNTPSConnectionStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} NNTPSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} NNTPSErrorEventParams;

typedef struct {
  const char* Group;
  int FirstArticle;
  int LastArticle;
  int CanPost;
  int reserved;
} NNTPSGroupListEventParams;

typedef struct {
  int ArticleNumber;
  const char* Subject;
  const char* From;
  const char* ArticleDate;
  const char* MessageId;
  const char* References;
  int ArticleSize;
  int ArticleLines;
  const char* OtherHeaders;
  int reserved;
} NNTPSGroupOverviewEventParams;

typedef struct {
  int ArticleNumber;
  const char* Header;
  int reserved;
} NNTPSGroupSearchEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} NNTPSHeaderEventParams;

typedef struct {
  int Direction;
  const char* Message;
  int reserved;
} NNTPSPITrailEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} NNTPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} NNTPSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} NNTPSStartTransferEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int EOL;
  int lenText;
  int reserved;
} NNTPSTransferEventParams;



class NNTPS {
  
  public: //events
  
    virtual int FireConnectionStatus(NNTPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireEndTransfer(NNTPSEndTransferEventParams *e) {return 0;}
    virtual int FireError(NNTPSErrorEventParams *e) {return 0;}
    virtual int FireGroupList(NNTPSGroupListEventParams *e) {return 0;}
    virtual int FireGroupOverview(NNTPSGroupOverviewEventParams *e) {return 0;}
    virtual int FireGroupSearch(NNTPSGroupSearchEventParams *e) {return 0;}
    virtual int FireHeader(NNTPSHeaderEventParams *e) {return 0;}
    virtual int FirePITrail(NNTPSPITrailEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(NNTPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(NNTPSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(NNTPSStartTransferEventParams *e) {return 0;}
    virtual int FireTransfer(NNTPSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL NNTPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((NNTPS*)lpObj)->NNTPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            NNTPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((NNTPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 2: {
            NNTPSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((NNTPS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 3: {
            NNTPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((NNTPS*)lpObj)->FireError(&e);
            break;
         }
         case 4: {
            NNTPSGroupListEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (int)IPS64CAST(param[3]),  0};
            ret_code = ((NNTPS*)lpObj)->FireGroupList(&e);
            break;
         }
         case 5: {
            NNTPSGroupOverviewEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (int)IPS64CAST(param[7]), (char*)IPS64CAST(param[8]),  0};
            ret_code = ((NNTPS*)lpObj)->FireGroupOverview(&e);
            break;
         }
         case 6: {
            NNTPSGroupSearchEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((NNTPS*)lpObj)->FireGroupSearch(&e);
            break;
         }
         case 7: {
            NNTPSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((NNTPS*)lpObj)->FireHeader(&e);
            break;
         }
         case 8: {
            NNTPSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((NNTPS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 9: {
            NNTPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((NNTPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 10: {
            NNTPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((NNTPS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 11: {
            NNTPSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((NNTPS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 12: {
            NNTPSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = ((NNTPS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int NNTPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    NNTPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_25) {
      m_pObj = IPWorksSSL_NNTPS_Create(NNTPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~NNTPS() {
      IPWorksSSL_NNTPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_NNTPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_NNTPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_NNTPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetArticleCount() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }

    inline char* GetArticleDate() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }


    inline char* GetArticleFrom() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 3, 0, 0);
      return (char*)val;
    }

    inline int SetArticleFrom(const char *lpArticleFrom) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 3, 0, (void*)lpArticleFrom, 0);
    }

    inline char* GetArticleHeaders() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 4, 0, 0);
      return (char*)val;
    }


    inline char* GetArticleId() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 5, 0, 0);
      return (char*)val;
    }


    inline int GetGroupOverviewCount() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 6, 0, 0);
      return (int)(long)val;
    }

    inline int GetGroupOverviewArticleLines(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 7, iGroupOverviewIndex, 0);
      return (int)(long)val;
    }

    inline int GetGroupOverviewArticleNumber(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 8, iGroupOverviewIndex, 0);
      return (int)(long)val;
    }

    inline int GetGroupOverviewArticleSize(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 9, iGroupOverviewIndex, 0);
      return (int)(long)val;
    }

    inline char* GetGroupOverviewDate(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 10, iGroupOverviewIndex, 0);
      return (char*)val;
    }


    inline char* GetGroupOverviewFrom(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 11, iGroupOverviewIndex, 0);
      return (char*)val;
    }


    inline char* GetGroupOverviewMessageId(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 12, iGroupOverviewIndex, 0);
      return (char*)val;
    }


    inline char* GetGroupOverviewOtherHeaders(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 13, iGroupOverviewIndex, 0);
      return (char*)val;
    }


    inline char* GetGroupOverviewReferences(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 14, iGroupOverviewIndex, 0);
      return (char*)val;
    }


    inline char* GetGroupOverviewSubject(int iGroupOverviewIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 15, iGroupOverviewIndex, 0);
      return (char*)val;
    }


    inline char* GetArticleReferences() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetArticleReferences(const char *lpArticleReferences) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 16, 0, (void*)lpArticleReferences, 0);
    }

    inline char* GetArticleReplyTo() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }

    inline int SetArticleReplyTo(const char *lpArticleReplyTo) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 17, 0, (void*)lpArticleReplyTo, 0);
    }

    inline char* GetArticleSubject() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetArticleSubject(const char *lpArticleSubject) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 18, 0, (void*)lpArticleSubject, 0);
    }

    inline char* GetArticleText() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetArticleText(const char *lpArticleText) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 19, 0, (void*)lpArticleText, 0);
    }

    inline char* GetAttachedFile() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }

    inline int SetAttachedFile(const char *lpAttachedFile) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 20, 0, (void*)lpAttachedFile, 0);
    }

    inline char* GetCheckDate() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetCheckDate(const char *lpCheckDate) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 21, 0, (void*)lpCheckDate, 0);
    }


    inline int SetCommand(const char *lpCommand) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 22, 0, (void*)lpCommand, 0);
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_NNTPS_Set(m_pObj, 23, 0, val, 0);
    }
    inline char* GetCurrentArticle() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }

    inline int SetCurrentArticle(const char *lpCurrentArticle) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 24, 0, (void*)lpCurrentArticle, 0);
    }

    inline char* GetCurrentGroup() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }

    inline int SetCurrentGroup(const char *lpCurrentGroup) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 25, 0, (void*)lpCurrentGroup, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 26, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_NNTPS_Set(m_pObj, 26, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 27, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_NNTPS_Set(m_pObj, 27, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 28, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 29, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 29, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 30, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_NNTPS_Set(m_pObj, 30, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 31, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 31, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetFirstArticle() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 32, 0, 0);
      return (int)(long)val;
    }

    inline int GetGroupListCount() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 33, 0, 0);
      return (int)(long)val;
    }

    inline int GetGroupListCanPost(int iGroupListIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 34, iGroupListIndex, 0);
      return (int)(long)val;
    }

    inline int GetGroupListFirstArticle(int iGroupListIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 35, iGroupListIndex, 0);
      return (int)(long)val;
    }

    inline char* GetGroupListGroup(int iGroupListIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 36, iGroupListIndex, 0);
      return (char*)val;
    }


    inline int GetGroupListLastArticle(int iGroupListIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 37, iGroupListIndex, 0);
      return (int)(long)val;
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 38, 0, 0);
      return (int)(long)val;
    }

    inline int GetLastArticle() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 39, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLastReply() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 40, 0, 0);
      return (char*)val;
    }


    inline char* GetLocalFile() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 41, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 41, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 42, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 42, 0, (void*)lpLocalHost, 0);
    }

    inline int GetMaxLines() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 43, 0, 0);
      return (int)(long)val;
    }
    inline int SetMaxLines(int lMaxLines) {
      void* val = (void*)IPS64CAST(lMaxLines);
      return IPWorksSSL_NNTPS_Set(m_pObj, 43, 0, val, 0);
    }
    inline char* GetNewsgroups() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 44, 0, 0);
      return (char*)val;
    }

    inline int SetNewsgroups(const char *lpNewsgroups) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 44, 0, (void*)lpNewsgroups, 0);
    }

    inline int GetNewsPort() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 45, 0, 0);
      return (int)(long)val;
    }
    inline int SetNewsPort(int lNewsPort) {
      void* val = (void*)IPS64CAST(lNewsPort);
      return IPWorksSSL_NNTPS_Set(m_pObj, 45, 0, val, 0);
    }
    inline char* GetNewsServer() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 46, 0, 0);
      return (char*)val;
    }

    inline int SetNewsServer(const char *lpNewsServer) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 46, 0, (void*)lpNewsServer, 0);
    }

    inline char* GetOrganization() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 47, 0, 0);
      return (char*)val;
    }

    inline int SetOrganization(const char *lpOrganization) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 47, 0, (void*)lpOrganization, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 48, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 48, 0, (void*)lpOtherHeaders, 0);
    }

    inline char* GetOverviewRange() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 49, 0, 0);
      return (char*)val;
    }

    inline int SetOverviewRange(const char *lpOverviewRange) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 49, 0, (void*)lpOverviewRange, 0);
    }

    inline int GetParsedHeaderCount() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 50, 0, 0);
      return (int)(long)val;
    }

    inline char* GetParsedHeaderField(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 51, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetParsedHeaderValue(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 52, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 53, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 53, 0, (void*)lpPassword, 0);
    }

    inline char* GetSearchHeader() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 54, 0, 0);
      return (char*)val;
    }

    inline int SetSearchHeader(const char *lpSearchHeader) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 54, 0, (void*)lpSearchHeader, 0);
    }

    inline char* GetSearchPattern() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 55, 0, 0);
      return (char*)val;
    }

    inline int SetSearchPattern(const char *lpSearchPattern) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 55, 0, (void*)lpSearchPattern, 0);
    }

    inline char* GetSearchRange() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 56, 0, 0);
      return (char*)val;
    }

    inline int SetSearchRange(const char *lpSearchRange) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 56, 0, (void*)lpSearchRange, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 57, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 57, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 58, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 58, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 59, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 59, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 60, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 60, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 61, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_NNTPS_Set(m_pObj, 61, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 62, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 62, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 63, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 64, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_NNTPS_Set(m_pObj, 64, 0, val, 0);
    }
    inline int GetTimeout() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 65, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_NNTPS_Set(m_pObj, 65, 0, val, 0);
    }
    inline char* GetUser() {
      void* val = IPWorksSSL_NNTPS_Get(m_pObj, 66, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 66, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_NNTPS_Do(m_pObj, 2, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline int FetchArticle() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int FetchArticleBody() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int FetchArticleHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int GroupOverview() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int GroupSearch() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 11, 0, param, cbparam);
      
      
    }
    inline int ListGroups() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 12, 0, param, cbparam);
      
      
    }
    inline int ListNewGroups() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 13, 0, param, cbparam);
      
      
    }
    inline char* LocalizeDate(const char* lpszDateTime) {
      void *param[1+1] = {(void*)IPS64CAST(lpszDateTime), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_NNTPS_Do(m_pObj, 14, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int PostArticle() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 15, 0, param, cbparam);
      
      
    }
    inline int ResetHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 16, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} NNTPSConnectionStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} NNTPSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} NNTPSErrorEventParamsW;

typedef struct {
  LPWSTR Group;
  int FirstArticle;
  int LastArticle;
  int CanPost;
  int reserved;
} NNTPSGroupListEventParamsW;

typedef struct {
  int ArticleNumber;
  LPWSTR Subject;
  LPWSTR From;
  LPWSTR ArticleDate;
  LPWSTR MessageId;
  LPWSTR References;
  int ArticleSize;
  int ArticleLines;
  LPWSTR OtherHeaders;
  int reserved;
} NNTPSGroupOverviewEventParamsW;

typedef struct {
  int ArticleNumber;
  LPWSTR Header;
  int reserved;
} NNTPSGroupSearchEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} NNTPSHeaderEventParamsW;

typedef struct {
  int Direction;
  LPWSTR Message;
  int reserved;
} NNTPSPITrailEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} NNTPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} NNTPSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} NNTPSStartTransferEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int EOL;
  int lenText;
  int reserved;
} NNTPSTransferEventParamsW;



class NNTPSW : public NNTPS {

  public: //properties
  


    inline LPWSTR GetArticleDate() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+2, 0, 0);
    }



    inline LPWSTR GetArticleFrom() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+3, 0, 0);
    }

    inline int SetArticleFrom(LPWSTR lpArticleFrom) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+3, 0, (void*)lpArticleFrom, 0);
    }

    inline LPWSTR GetArticleHeaders() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+4, 0, 0);
    }



    inline LPWSTR GetArticleId() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+5, 0, 0);
    }











    inline LPWSTR GetGroupOverviewDate(int iGroupOverviewIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+10, iGroupOverviewIndex, 0);
    }



    inline LPWSTR GetGroupOverviewFrom(int iGroupOverviewIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+11, iGroupOverviewIndex, 0);
    }



    inline LPWSTR GetGroupOverviewMessageId(int iGroupOverviewIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+12, iGroupOverviewIndex, 0);
    }



    inline LPWSTR GetGroupOverviewOtherHeaders(int iGroupOverviewIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+13, iGroupOverviewIndex, 0);
    }



    inline LPWSTR GetGroupOverviewReferences(int iGroupOverviewIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+14, iGroupOverviewIndex, 0);
    }



    inline LPWSTR GetGroupOverviewSubject(int iGroupOverviewIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+15, iGroupOverviewIndex, 0);
    }



    inline LPWSTR GetArticleReferences() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetArticleReferences(LPWSTR lpArticleReferences) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+16, 0, (void*)lpArticleReferences, 0);
    }

    inline LPWSTR GetArticleReplyTo() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetArticleReplyTo(LPWSTR lpArticleReplyTo) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+17, 0, (void*)lpArticleReplyTo, 0);
    }

    inline LPWSTR GetArticleSubject() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetArticleSubject(LPWSTR lpArticleSubject) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+18, 0, (void*)lpArticleSubject, 0);
    }

    inline LPWSTR GetArticleText() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetArticleText(LPWSTR lpArticleText) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+19, 0, (void*)lpArticleText, 0);
    }

    inline LPWSTR GetAttachedFile() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+20, 0, 0);
    }

    inline int SetAttachedFile(LPWSTR lpAttachedFile) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+20, 0, (void*)lpAttachedFile, 0);
    }

    inline LPWSTR GetCheckDate() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetCheckDate(LPWSTR lpCheckDate) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+21, 0, (void*)lpCheckDate, 0);
    }



    inline int SetCommand(LPWSTR lpCommand) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+22, 0, (void*)lpCommand, 0);
    }



    inline LPWSTR GetCurrentArticle() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetCurrentArticle(LPWSTR lpCurrentArticle) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+24, 0, (void*)lpCurrentArticle, 0);
    }

    inline LPWSTR GetCurrentGroup() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetCurrentGroup(LPWSTR lpCurrentGroup) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+25, 0, (void*)lpCurrentGroup, 0);
    }





    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+28, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+28, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+29, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+29, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+31, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+31, 0, (void*)lpFirewallUser, 0);
    }









    inline LPWSTR GetGroupListGroup(int iGroupListIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+36, iGroupListIndex, 0);
    }









    inline LPWSTR GetLastReply() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+40, 0, 0);
    }



    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+10000+41, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+10000+41, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+42, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+42, 0, (void*)lpLocalHost, 0);
    }



    inline LPWSTR GetNewsgroups() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+44, 0, 0);
    }

    inline int SetNewsgroups(LPWSTR lpNewsgroups) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+44, 0, (void*)lpNewsgroups, 0);
    }



    inline LPWSTR GetNewsServer() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+46, 0, 0);
    }

    inline int SetNewsServer(LPWSTR lpNewsServer) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+46, 0, (void*)lpNewsServer, 0);
    }

    inline LPWSTR GetOrganization() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+47, 0, 0);
    }

    inline int SetOrganization(LPWSTR lpOrganization) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+47, 0, (void*)lpOrganization, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+48, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+48, 0, (void*)lpOtherHeaders, 0);
    }

    inline LPWSTR GetOverviewRange() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+49, 0, 0);
    }

    inline int SetOverviewRange(LPWSTR lpOverviewRange) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+49, 0, (void*)lpOverviewRange, 0);
    }



    inline LPWSTR GetParsedHeaderField(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+51, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetParsedHeaderValue(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+52, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+53, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+53, 0, (void*)lpPassword, 0);
    }

    inline LPWSTR GetSearchHeader() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+54, 0, 0);
    }

    inline int SetSearchHeader(LPWSTR lpSearchHeader) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+54, 0, (void*)lpSearchHeader, 0);
    }

    inline LPWSTR GetSearchPattern() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+55, 0, 0);
    }

    inline int SetSearchPattern(LPWSTR lpSearchPattern) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+55, 0, (void*)lpSearchPattern, 0);
    }

    inline LPWSTR GetSearchRange() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+56, 0, 0);
    }

    inline int SetSearchRange(LPWSTR lpSearchRange) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+56, 0, (void*)lpSearchRange, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+57, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+57, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 57, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 57, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+58, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+58, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 58, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 58, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+59, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+59, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 59, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 59, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+60, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+60, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+62, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+62, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+63, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_NNTPS_Get(m_pObj, 63, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }





    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_NNTPS_Get(m_pObj, 10000+66, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_NNTPS_Set(m_pObj, 10000+66, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnectionStatus(NNTPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(NNTPSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(NNTPSErrorEventParamsW *e) {return 0;}
    virtual int FireGroupList(NNTPSGroupListEventParamsW *e) {return 0;}
    virtual int FireGroupOverview(NNTPSGroupOverviewEventParamsW *e) {return 0;}
    virtual int FireGroupSearch(NNTPSGroupSearchEventParamsW *e) {return 0;}
    virtual int FireHeader(NNTPSHeaderEventParamsW *e) {return 0;}
    virtual int FirePITrail(NNTPSPITrailEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(NNTPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(NNTPSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(NNTPSStartTransferEventParamsW *e) {return 0;}
    virtual int FireTransfer(NNTPSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int NNTPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            NNTPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 2: {
            NNTPSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 3: {
            NNTPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 4: {
            NNTPSGroupListEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (int)IPS64CAST(param[3]),  0};
            ret_code = FireGroupList(&e);
            break;
         }
         case 5: {
            NNTPSGroupOverviewEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (int)IPS64CAST(param[7]), (LPWSTR)IPS64CAST(param[8]),  0};
            ret_code = FireGroupOverview(&e);
            break;
         }
         case 6: {
            NNTPSGroupSearchEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireGroupSearch(&e);
            break;
         }
         case 7: {
            NNTPSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 8: {
            NNTPSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 9: {
            NNTPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 10: {
            NNTPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 11: {
            NNTPSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 12: {
            NNTPSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnectionStatus(NNTPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireEndTransfer(NNTPSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(NNTPSErrorEventParams *e) {return -10000;}
    virtual int FireGroupList(NNTPSGroupListEventParams *e) {return -10000;}
    virtual int FireGroupOverview(NNTPSGroupOverviewEventParams *e) {return -10000;}
    virtual int FireGroupSearch(NNTPSGroupSearchEventParams *e) {return -10000;}
    virtual int FireHeader(NNTPSHeaderEventParams *e) {return -10000;}
    virtual int FirePITrail(NNTPSPITrailEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(NNTPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(NNTPSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(NNTPSStartTransferEventParams *e) {return -10000;}
    virtual int FireTransfer(NNTPSTransferEventParams *e) {return -10000;}

  public: //methods

    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_NNTPS_Do(m_pObj, 10000+2, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline int FetchArticle() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int FetchArticleBody() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int FetchArticleHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int GroupOverview() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int GroupSearch() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+11, 0, param, cbparam);
      
    }
    inline int ListGroups() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+12, 0, param, cbparam);
      
    }
    inline int ListNewGroups() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+13, 0, param, cbparam);
      
    }
    inline LPWSTR LocalizeDate(LPWSTR lpszDateTime) {
      void *param[1+1] = {(void*)lpszDateTime, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_NNTPS_Do(m_pObj, 10000+14, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int PostArticle() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+15, 0, param, cbparam);
      
    }
    inline int ResetHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_NNTPS_Do(m_pObj, 10000+16, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_NNTPS_H_




