/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_SMPPS_H_
#define _IPWORKSSSL_SMPPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TSMPPMessagePriorities
#define SMPP_MESSAGE_PRIORITY_LOW                          0
#define SMPP_MESSAGE_PRIORITY_NORMAL                       1
#define SMPP_MESSAGE_PRIORITY_HIGH                         2
#define SMPP_MESSAGE_PRIORITY_URGENT                       3

//TSMPPProtocols
#define SMPP_SMPP                                          0
#define SMPP_CIMD2                                         1

//SMPPRecipientTypes
#define SMPP_RECIPIENT_TYPE_NORMAL                         0
#define SMPP_RECIPIENT_TYPE_LIST                           1

//TSMPPServiceTypes
#define SMPP_SERVICE_DEFAULT                               0
#define SMPP_SERVICE_CMT                                   1
#define SMPP_SERVICE_CPT                                   2
#define SMPP_SERVICE_VMN                                   3
#define SMPP_SERVICE_VMA                                   4
#define SMPP_SERVICE_WAP                                   5
#define SMPP_SERVICE_USSD                                  6
#define SMPP_SERVICE_CBS                                   7

//TSMPPVersions
#define SMPP_VERSION_50                                    0
#define SMPP_VERSION_34                                    1
#define SMPP_VERSION_33                                    2

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_SMPPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SMPPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SMPPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_SMPPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SMPPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SMPPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_SMPPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SMPPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SMPPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SMPPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} SMPPSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} SMPPSConnectionStatusEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} SMPPSDisconnectedEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} SMPPSErrorEventParams;

typedef struct {
  const char* SourceAddress;
  const char* ScheduleDeliveryTime;
  const char* ValidityPeriod;
  const char* Message;
  int MessagePart;
  int MessagePartCount;
  int MessagePartReference;
  const char* DestinationAddress;
  int ResponseErrorCode;
  int reserved;
} SMPPSMessageInEventParams;

typedef struct {
  const char* MessageId;
  int MessageState;
  int MessageError;
  const char* FinalDate;
  int reserved;
} SMPPSMessageStatusEventParams;

typedef struct {
  int Direction;
  const char* PDU;
  int CommandLength;
  int CommandId;
  const char* CommandDescription;
  const char* CommandStatus;
  int SequenceNumber;
  int lenPDU;
  int reserved;
} SMPPSPITrailEventParams;

typedef struct {
  int reserved;
} SMPPSReadyToSendEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} SMPPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} SMPPSSSLStatusEventParams;



class SMPPS {
  
  public: //events
  
    virtual int FireConnected(SMPPSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(SMPPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDisconnected(SMPPSDisconnectedEventParams *e) {return 0;}
    virtual int FireError(SMPPSErrorEventParams *e) {return 0;}
    virtual int FireMessageIn(SMPPSMessageInEventParams *e) {return 0;}
    virtual int FireMessageStatus(SMPPSMessageStatusEventParams *e) {return 0;}
    virtual int FirePITrail(SMPPSPITrailEventParams *e) {return 0;}
    virtual int FireReadyToSend(SMPPSReadyToSendEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(SMPPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(SMPPSSSLStatusEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL SMPPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((SMPPS*)lpObj)->SMPPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            SMPPSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SMPPS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            SMPPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((SMPPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 3: {
            SMPPSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SMPPS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 4: {
            SMPPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SMPPS*)lpObj)->FireError(&e);
            break;
         }
         case 5: {
            SMPPSMessageInEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (char*)IPS64CAST(param[7]), (int)IPS64CAST(param[8]),  0};
            ret_code = ((SMPPS*)lpObj)->FireMessageIn(&e);
            param[8] = (void*)IPS64CAST(e.ResponseErrorCode);
            break;
         }
         case 6: {
            SMPPSMessageStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]),  0};
            ret_code = ((SMPPS*)lpObj)->FireMessageStatus(&e);
            break;
         }
         case 7: {
            SMPPSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (int)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (int)IPS64CAST(cbparam[1]),  0};
            ret_code = ((SMPPS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 8: {
            SMPPSReadyToSendEventParams e = { 0};
            ret_code = ((SMPPS*)lpObj)->FireReadyToSend(&e);
            break;
         }
         case 9: {
            SMPPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((SMPPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 10: {
            SMPPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((SMPPS*)lpObj)->FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int SMPPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    SMPPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_71) {
      m_pObj = IPWorksSSL_SMPPS_Create(SMPPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~SMPPS() {
      IPWorksSSL_SMPPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_SMPPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_SMPPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_SMPPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetConnected() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 2, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_SMPPS_Set(m_pObj, 2, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_SMPPS_Set(m_pObj, 3, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 4, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 4, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 5, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 5, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 6, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_SMPPS_Set(m_pObj, 6, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 7, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 7, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 8, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 9, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetMessageExpiration() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 10, 0, 0);
      return (char*)val;
    }

    inline int SetMessageExpiration(const char *lpMessageExpiration) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10, 0, (void*)lpMessageExpiration, 0);
    }

    inline char* GetMessageId() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 11, 0, 0);
      return (char*)val;
    }


    inline int GetMessagePriority() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 12, 0, 0);
      return (int)(long)val;
    }
    inline int SetMessagePriority(int iMessagePriority) {
      void* val = (void*)IPS64CAST(iMessagePriority);
      return IPWorksSSL_SMPPS_Set(m_pObj, 12, 0, val, 0);
    }
    inline char* GetPassword() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 13, 0, (void*)lpPassword, 0);
    }

    inline int GetProtocol() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 14, 0, 0);
      return (int)(long)val;
    }
    inline int SetProtocol(int iProtocol) {
      void* val = (void*)IPS64CAST(iProtocol);
      return IPWorksSSL_SMPPS_Set(m_pObj, 14, 0, val, 0);
    }
    inline int GetRecipientCount() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 15, 0, 0);
      return (int)(long)val;
    }
    inline int SetRecipientCount(int iRecipientCount) {
      void* val = (void*)IPS64CAST(iRecipientCount);
      return IPWorksSSL_SMPPS_Set(m_pObj, 15, 0, val, 0);
    }
    inline char* GetRecipientAddress(int iRecipientIndex) {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 16, iRecipientIndex, 0);
      return (char*)val;
    }

    inline int SetRecipientAddress(int iRecipientIndex, const char *lpRecipientAddress) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 16, iRecipientIndex, (void*)lpRecipientAddress, 0);
    }

    inline int GetRecipientType(int iRecipientIndex) {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 17, iRecipientIndex, 0);
      return (int)(long)val;
    }
    inline int SetRecipientType(int iRecipientIndex, int iRecipientType) {
      void* val = (void*)IPS64CAST(iRecipientType);
      return IPWorksSSL_SMPPS_Set(m_pObj, 17, iRecipientIndex, val, 0);
    }
    inline char* GetScheduledDelivery() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetScheduledDelivery(const char *lpScheduledDelivery) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 18, 0, (void*)lpScheduledDelivery, 0);
    }

    inline char* GetSenderAddress() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetSenderAddress(const char *lpSenderAddress) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 19, 0, (void*)lpSenderAddress, 0);
    }

    inline int GetServiceType() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 20, 0, 0);
      return (int)(long)val;
    }
    inline int SetServiceType(int iServiceType) {
      void* val = (void*)IPS64CAST(iServiceType);
      return IPWorksSSL_SMPPS_Set(m_pObj, 20, 0, val, 0);
    }
    inline int GetSMPPPort() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 21, 0, 0);
      return (int)(long)val;
    }
    inline int SetSMPPPort(int iSMPPPort) {
      void* val = (void*)IPS64CAST(iSMPPPort);
      return IPWorksSSL_SMPPS_Set(m_pObj, 21, 0, val, 0);
    }
    inline char* GetSMPPServer() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetSMPPServer(const char *lpSMPPServer) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 22, 0, (void*)lpSMPPServer, 0);
    }

    inline int GetSMPPVersion() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }
    inline int SetSMPPVersion(int iSMPPVersion) {
      void* val = (void*)IPS64CAST(iSMPPVersion);
      return IPWorksSSL_SMPPS_Set(m_pObj, 23, 0, val, 0);
    }
    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 24, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 24, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 25, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 25, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 26, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 26, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 27, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 27, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 28, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_SMPPS_Set(m_pObj, 28, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 29, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 29, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 30, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 31, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_SMPPS_Set(m_pObj, 31, 0, val, 0);
    }
    inline char* GetSystemType() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 32, 0, 0);
      return (char*)val;
    }

    inline int SetSystemType(const char *lpSystemType) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 32, 0, (void*)lpSystemType, 0);
    }

    inline int GetTimeout() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 33, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_SMPPS_Set(m_pObj, 33, 0, val, 0);
    }
    inline char* GetUserId() {
      void* val = IPWorksSSL_SMPPS_Get(m_pObj, 34, 0, 0);
      return (char*)val;
    }

    inline int SetUserId(const char *lpUserId) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 34, 0, (void*)lpUserId, 0);
    }


  public: //methods

    inline int AddRecipient(int iRecipientType, const char* lpszRecipientAddress) {
      void *param[2+1] = {(void*)IPS64CAST(iRecipientType), (void*)IPS64CAST(lpszRecipientAddress), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 2, 2, param, cbparam);
      
      
    }
    inline int CancelMessage(const char* lpszMessageId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszMessageId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 3, 1, param, cbparam);
      
      
    }
    inline int CheckLink() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int CheckMessageStatus(const char* lpszMessageId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszMessageId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 5, 1, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 6, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect(const char* lpszUserId, const char* lpszPassword) {
      void *param[2+1] = {(void*)IPS64CAST(lpszUserId), (void*)IPS64CAST(lpszPassword), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 7, 2, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline int ReplaceMessage(const char* lpszMessageId, const char* lpszNewMessage) {
      void *param[2+1] = {(void*)IPS64CAST(lpszMessageId), (void*)IPS64CAST(lpszNewMessage), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 11, 2, param, cbparam);
      
      
    }
    inline char* SendCommand(int iCommandId, const char* lpPayload, int lenPayload, int *lpSize = 0) {
      void *param[2+1] = {(void*)IPS64CAST(iCommandId), (void*)IPS64CAST(lpPayload), 0};
      int cbparam[2+1] = {0, lenPayload, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 12, 2, param, cbparam);
      if (lpSize) *lpSize = cbparam[2];
      return (char*)IPS64CAST(param[2]);
    }
    inline char* SendData(const char* lpData, int lenData) {
      void *param[1+1] = {(void*)IPS64CAST(lpData), 0};
      int cbparam[1+1] = {lenData, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 13, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline char* SendMessage(const char* lpszMessage) {
      void *param[1+1] = {(void*)IPS64CAST(lpszMessage), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 14, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SMPPSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SMPPSConnectionStatusEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SMPPSDisconnectedEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} SMPPSErrorEventParamsW;

typedef struct {
  LPWSTR SourceAddress;
  LPWSTR ScheduleDeliveryTime;
  LPWSTR ValidityPeriod;
  LPWSTR Message;
  int MessagePart;
  int MessagePartCount;
  int MessagePartReference;
  LPWSTR DestinationAddress;
  int ResponseErrorCode;
  int reserved;
} SMPPSMessageInEventParamsW;

typedef struct {
  LPWSTR MessageId;
  int MessageState;
  int MessageError;
  LPWSTR FinalDate;
  int reserved;
} SMPPSMessageStatusEventParamsW;

typedef struct {
  int Direction;
  LPWSTR PDU;
  int CommandLength;
  int CommandId;
  LPWSTR CommandDescription;
  LPWSTR CommandStatus;
  int SequenceNumber;
  int lenPDU;
  int reserved;
} SMPPSPITrailEventParamsW;

typedef struct {
  int reserved;
} SMPPSReadyToSendEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} SMPPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} SMPPSSSLStatusEventParamsW;



class SMPPSW : public SMPPS {

  public: //properties
  






    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+4, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+4, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+5, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+5, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+7, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+7, 0, (void*)lpFirewallUser, 0);
    }



    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+9, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetMessageExpiration() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+10, 0, 0);
    }

    inline int SetMessageExpiration(LPWSTR lpMessageExpiration) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+10, 0, (void*)lpMessageExpiration, 0);
    }

    inline LPWSTR GetMessageId() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+11, 0, 0);
    }





    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+13, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+13, 0, (void*)lpPassword, 0);
    }





    inline LPWSTR GetRecipientAddress(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+16, iRecipientIndex, 0);
    }

    inline int SetRecipientAddress(int iRecipientIndex, LPWSTR lpRecipientAddress) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+16, iRecipientIndex, (void*)lpRecipientAddress, 0);
    }



    inline LPWSTR GetScheduledDelivery() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetScheduledDelivery(LPWSTR lpScheduledDelivery) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+18, 0, (void*)lpScheduledDelivery, 0);
    }

    inline LPWSTR GetSenderAddress() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetSenderAddress(LPWSTR lpSenderAddress) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+19, 0, (void*)lpSenderAddress, 0);
    }





    inline LPWSTR GetSMPPServer() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetSMPPServer(LPWSTR lpSMPPServer) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+22, 0, (void*)lpSMPPServer, 0);
    }



    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+24, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 24, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 24, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+25, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 25, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 25, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+26, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+26, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 26, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 26, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+27, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+27, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+29, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+29, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+30, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_SMPPS_Get(m_pObj, 30, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }



    inline LPWSTR GetSystemType() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+32, 0, 0);
    }

    inline int SetSystemType(LPWSTR lpSystemType) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+32, 0, (void*)lpSystemType, 0);
    }



    inline LPWSTR GetUserId() {
      return (LPWSTR)IPWorksSSL_SMPPS_Get(m_pObj, 10000+34, 0, 0);
    }

    inline int SetUserId(LPWSTR lpUserId) {
      return IPWorksSSL_SMPPS_Set(m_pObj, 10000+34, 0, (void*)lpUserId, 0);
    }



  public: //events
  
    virtual int FireConnected(SMPPSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(SMPPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDisconnected(SMPPSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireError(SMPPSErrorEventParamsW *e) {return 0;}
    virtual int FireMessageIn(SMPPSMessageInEventParamsW *e) {return 0;}
    virtual int FireMessageStatus(SMPPSMessageStatusEventParamsW *e) {return 0;}
    virtual int FirePITrail(SMPPSPITrailEventParamsW *e) {return 0;}
    virtual int FireReadyToSend(SMPPSReadyToSendEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(SMPPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(SMPPSSSLStatusEventParamsW *e) {return 0;}


  protected:
  
    virtual int SMPPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            SMPPSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            SMPPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 3: {
            SMPPSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 4: {
            SMPPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 5: {
            SMPPSMessageInEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (LPWSTR)IPS64CAST(param[7]), (int)IPS64CAST(param[8]),  0};
            ret_code = FireMessageIn(&e);
            param[8] = (void*)(e.ResponseErrorCode);
            break;
         }
         case 6: {
            SMPPSMessageStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]),  0};
            ret_code = FireMessageStatus(&e);
            break;
         }
         case 7: {
            SMPPSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (int)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (int)IPS64CAST(cbparam[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 8: {
            SMPPSReadyToSendEventParamsW e = { 0};
            ret_code = FireReadyToSend(&e);
            break;
         }
         case 9: {
            SMPPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 10: {
            SMPPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(SMPPSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(SMPPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDisconnected(SMPPSDisconnectedEventParams *e) {return -10000;}
    virtual int FireError(SMPPSErrorEventParams *e) {return -10000;}
    virtual int FireMessageIn(SMPPSMessageInEventParams *e) {return -10000;}
    virtual int FireMessageStatus(SMPPSMessageStatusEventParams *e) {return -10000;}
    virtual int FirePITrail(SMPPSPITrailEventParams *e) {return -10000;}
    virtual int FireReadyToSend(SMPPSReadyToSendEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(SMPPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(SMPPSSSLStatusEventParams *e) {return -10000;}

  public: //methods

    inline int AddRecipient(int iRecipientType, LPWSTR lpszRecipientAddress) {
      void *param[2+1] = {(void*)iRecipientType, (void*)lpszRecipientAddress, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+2, 2, param, cbparam);
      
    }
    inline int CancelMessage(LPWSTR lpszMessageId) {
      void *param[1+1] = {(void*)lpszMessageId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+3, 1, param, cbparam);
      
    }
    inline int CheckLink() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int CheckMessageStatus(LPWSTR lpszMessageId) {
      void *param[1+1] = {(void*)lpszMessageId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+5, 1, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 10000+6, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect(LPWSTR lpszUserId, LPWSTR lpszPassword) {
      void *param[2+1] = {(void*)lpszUserId, (void*)lpszPassword, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+7, 2, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline int ReplaceMessage(LPWSTR lpszMessageId, LPWSTR lpszNewMessage) {
      void *param[2+1] = {(void*)lpszMessageId, (void*)lpszNewMessage, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SMPPS_Do(m_pObj, 10000+11, 2, param, cbparam);
      
    }
    inline LPWSTR SendCommand(int iCommandId, LPWSTR lpPayload, int lenPayload) {
      void *param[2+1] = {(void*)iCommandId, (void*)lpPayload, 0};
      int cbparam[2+1] = {0, lenPayload, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 10000+12, 2, param, cbparam);
      return (LPWSTR)IPS64CAST(param[2]);
    }
    inline LPWSTR SendData(LPWSTR lpData, int lenData) {
      void *param[1+1] = {(void*)lpData, 0};
      int cbparam[1+1] = {lenData, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 10000+13, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline LPWSTR SendMessage(LPWSTR lpszMessage) {
      void *param[1+1] = {(void*)lpszMessage, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SMPPS_Do(m_pObj, 10000+14, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_SMPPS_H_




