/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_WEBDAVS_H_
#define _IPWORKSSSL_WEBDAVS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthSchemes
#define AUTH_BASIC                                         0
#define AUTH_DIGEST                                        1
#define AUTH_PROPRIETARY                                   2
#define AUTH_NONE                                          3
#define AUTH_NTLM                                          4
#define AUTH_NEGOTIATE                                     5
#define AUTH_OAUTH                                         6

//TDepth
#define DP_UNSPECIFIED                                     0
#define DP_RESOURCE_ONLY                                   1
#define DP_IMMEDIATE_CHILDREN                              2
#define DP_INFINITY                                        3

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TFollowRedirects
#define FR_NEVER                                           0
#define FR_ALWAYS                                          1
#define FR_SAME_SCHEME                                     2

//DAVPropertyOperations
#define OP_NONE                                            0
#define OP_SET                                             1
#define OP_DELETE                                          2

//ProxySSLTypes
#define PS_AUTOMATIC                                       0
#define PS_ALWAYS                                          1
#define PS_NEVER                                           2
#define PS_TUNNEL                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_WebDAVS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebDAVS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebDAVS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_WebDAVS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebDAVS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebDAVS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_WebDAVS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebDAVS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebDAVS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebDAVS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} WebDAVSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} WebDAVSConnectionStatusEventParams;

typedef struct {
  const char* ResourceURI;
  const char* DisplayName;
  const char* ContentLanguage;
  const char* ContentLength;
  const char* ContentType;
  const char* LastModified;
  int reserved;
} WebDAVSDirListEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} WebDAVSDisconnectedEventParams;

typedef struct {
  int Direction;
  int reserved;
} WebDAVSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} WebDAVSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} WebDAVSHeaderEventParams;

typedef struct {
  const char* Location;
  int Accept;
  int reserved;
} WebDAVSRedirectEventParams;

typedef struct {
  const char* ResourceURI;
  const char* ResourceProperties;
  int reserved;
} WebDAVSResourcePropertiesEventParams;

typedef struct {
  const char* Name;
  const char* Value;
  const char* Expires;
  const char* Domain;
  const char* Path;
  int Secure;
  int reserved;
} WebDAVSSetCookieEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} WebDAVSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} WebDAVSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} WebDAVSStartTransferEventParams;

typedef struct {
  const char* HTTPVersion;
  int StatusCode;
  const char* Description;
  int reserved;
} WebDAVSStatusEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int lenText;
  int reserved;
} WebDAVSTransferEventParams;



class WebDAVS {
  
  public: //events
  
    virtual int FireConnected(WebDAVSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(WebDAVSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDirList(WebDAVSDirListEventParams *e) {return 0;}
    virtual int FireDisconnected(WebDAVSDisconnectedEventParams *e) {return 0;}
    virtual int FireEndTransfer(WebDAVSEndTransferEventParams *e) {return 0;}
    virtual int FireError(WebDAVSErrorEventParams *e) {return 0;}
    virtual int FireHeader(WebDAVSHeaderEventParams *e) {return 0;}
    virtual int FireRedirect(WebDAVSRedirectEventParams *e) {return 0;}
    virtual int FireResourceProperties(WebDAVSResourcePropertiesEventParams *e) {return 0;}
    virtual int FireSetCookie(WebDAVSSetCookieEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(WebDAVSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(WebDAVSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(WebDAVSStartTransferEventParams *e) {return 0;}
    virtual int FireStatus(WebDAVSStatusEventParams *e) {return 0;}
    virtual int FireTransfer(WebDAVSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL WebDAVSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((WebDAVS*)lpObj)->WebDAVSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            WebDAVSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            WebDAVSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 3: {
            WebDAVSDirListEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireDirList(&e);
            break;
         }
         case 4: {
            WebDAVSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 5: {
            WebDAVSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 6: {
            WebDAVSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireError(&e);
            break;
         }
         case 7: {
            WebDAVSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireHeader(&e);
            break;
         }
         case 8: {
            WebDAVSRedirectEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireRedirect(&e);
            param[1] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 9: {
            WebDAVSResourcePropertiesEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireResourceProperties(&e);
            break;
         }
         case 10: {
            WebDAVSSetCookieEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireSetCookie(&e);
            break;
         }
         case 11: {
            WebDAVSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 12: {
            WebDAVSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 13: {
            WebDAVSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 14: {
            WebDAVSStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireStatus(&e);
            break;
         }
         case 15: {
            WebDAVSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = ((WebDAVS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int WebDAVSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    WebDAVS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_61) {
      m_pObj = IPWorksSSL_WebDAVS_Create(WebDAVSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~WebDAVS() {
      IPWorksSSL_WebDAVS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_WebDAVS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_WebDAVS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAccept() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAccept(const char *lpAccept) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 1, 0, (void*)lpAccept, 0);
    }

    inline char* GetAuthorization() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }

    inline int SetAuthorization(const char *lpAuthorization) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 2, 0, (void*)lpAuthorization, 0);
    }

    inline int GetAuthScheme() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthScheme(int iAuthScheme) {
      void* val = (void*)IPS64CAST(iAuthScheme);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 3, 0, val, 0);
    }
    inline int GetConnected() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 4, 0, val, 0);
    }
    inline int GetCookieCount() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 5, 0, 0);
      return (int)(long)val;
    }
    inline int SetCookieCount(int iCookieCount) {
      void* val = (void*)IPS64CAST(iCookieCount);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 5, 0, val, 0);
    }
    inline char* GetCookieDomain(int iCookieIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 6, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieExpiration(int iCookieIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 7, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieName(int iCookieIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 8, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieName(int iCookieIndex, const char *lpCookieName) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 8, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline char* GetCookiePath(int iCookieIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 9, iCookieIndex, 0);
      return (char*)val;
    }


    inline int GetCookieSecure(int iCookieIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 10, iCookieIndex, 0);
      return (int)(long)val;
    }

    inline char* GetCookieValue(int iCookieIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 11, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieValue(int iCookieIndex, const char *lpCookieValue) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 11, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline int GetDepth() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 12, 0, 0);
      return (int)(long)val;
    }
    inline int SetDepth(int iDepth) {
      void* val = (void*)IPS64CAST(iDepth);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 12, 0, val, 0);
    }
    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 13, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 13, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 14, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 14, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 15, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 16, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 17, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 17, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 18, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetFollowRedirects() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 19, 0, 0);
      return (int)(long)val;
    }
    inline int SetFollowRedirects(int iFollowRedirects) {
      void* val = (void*)IPS64CAST(iFollowRedirects);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 19, 0, val, 0);
    }
    inline int GetIdle() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 20, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLocalFile() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 21, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 22, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetLockOwner() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 23, 0, 0);
      return (char*)val;
    }

    inline int SetLockOwner(const char *lpLockOwner) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 23, 0, (void*)lpLockOwner, 0);
    }

    inline char* GetLockScope() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }

    inline int SetLockScope(const char *lpLockScope) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 24, 0, (void*)lpLockScope, 0);
    }

    inline int GetLockTimeout() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 25, 0, 0);
      return (int)(long)val;
    }
    inline int SetLockTimeout(int iLockTimeout) {
      void* val = (void*)IPS64CAST(iLockTimeout);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 25, 0, val, 0);
    }
    inline char* GetLockTokens() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }

    inline int SetLockTokens(const char *lpLockTokens) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 26, 0, (void*)lpLockTokens, 0);
    }

    inline char* GetLockType() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 27, 0, 0);
      return (char*)val;
    }

    inline int SetLockType(const char *lpLockType) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 27, 0, (void*)lpLockType, 0);
    }

    inline int GetNamespaceCount() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 28, 0, 0);
      return (int)(long)val;
    }
    inline int SetNamespaceCount(int iNamespaceCount) {
      void* val = (void*)IPS64CAST(iNamespaceCount);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 28, 0, val, 0);
    }
    inline char* GetNamespacePrefix(int iNamespaceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 29, iNamespaceIndex, 0);
      return (char*)val;
    }

    inline int SetNamespacePrefix(int iNamespaceIndex, const char *lpNamespacePrefix) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 29, iNamespaceIndex, (void*)lpNamespacePrefix, 0);
    }

    inline char* GetNamespaceURI(int iNamespaceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 30, iNamespaceIndex, 0);
      return (char*)val;
    }

    inline int SetNamespaceURI(int iNamespaceIndex, const char *lpNamespaceURI) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 30, iNamespaceIndex, (void*)lpNamespaceURI, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 31, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 31, 0, (void*)lpOtherHeaders, 0);
    }

    inline int GetParsedHeaderCount() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 32, 0, 0);
      return (int)(long)val;
    }

    inline char* GetParsedHeaderField(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 33, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetParsedHeaderValue(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 34, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 35, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 35, 0, (void*)lpPassword, 0);
    }

    inline int GetPropertyCount() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 36, 0, 0);
      return (int)(long)val;
    }
    inline int SetPropertyCount(int iPropertyCount) {
      void* val = (void*)IPS64CAST(iPropertyCount);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 36, 0, val, 0);
    }
    inline char* GetPropertyAttr(int iPropertyIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 37, iPropertyIndex, 0);
      return (char*)val;
    }

    inline int SetPropertyAttr(int iPropertyIndex, const char *lpPropertyAttr) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 37, iPropertyIndex, (void*)lpPropertyAttr, 0);
    }

    inline char* GetPropertyName(int iPropertyIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 38, iPropertyIndex, 0);
      return (char*)val;
    }

    inline int SetPropertyName(int iPropertyIndex, const char *lpPropertyName) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 38, iPropertyIndex, (void*)lpPropertyName, 0);
    }

    inline char* GetPropertyNamespaceURI(int iPropertyIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 39, iPropertyIndex, 0);
      return (char*)val;
    }

    inline int SetPropertyNamespaceURI(int iPropertyIndex, const char *lpPropertyNamespaceURI) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 39, iPropertyIndex, (void*)lpPropertyNamespaceURI, 0);
    }

    inline int GetPropertyOperation(int iPropertyIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 40, iPropertyIndex, 0);
      return (int)(long)val;
    }
    inline int SetPropertyOperation(int iPropertyIndex, int iPropertyOperation) {
      void* val = (void*)IPS64CAST(iPropertyOperation);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 40, iPropertyIndex, val, 0);
    }
    inline char* GetPropertyStatus(int iPropertyIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 41, iPropertyIndex, 0);
      return (char*)val;
    }


    inline char* GetPropertyValue(int iPropertyIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 42, iPropertyIndex, 0);
      return (char*)val;
    }

    inline int SetPropertyValue(int iPropertyIndex, const char *lpPropertyValue) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 42, iPropertyIndex, (void*)lpPropertyValue, 0);
    }

    inline int GetProxyAuthScheme() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 43, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAuthScheme(int iProxyAuthScheme) {
      void* val = (void*)IPS64CAST(iProxyAuthScheme);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 43, 0, val, 0);
    }
    inline int GetProxyAutoDetect() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 44, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAutoDetect(int bProxyAutoDetect) {
      void* val = (void*)IPS64CAST(bProxyAutoDetect);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 44, 0, val, 0);
    }
    inline char* GetProxyPassword() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 45, 0, 0);
      return (char*)val;
    }

    inline int SetProxyPassword(const char *lpProxyPassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 45, 0, (void*)lpProxyPassword, 0);
    }

    inline int GetProxyPort() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 46, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyPort(int lProxyPort) {
      void* val = (void*)IPS64CAST(lProxyPort);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 46, 0, val, 0);
    }
    inline char* GetProxyServer() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 47, 0, 0);
      return (char*)val;
    }

    inline int SetProxyServer(const char *lpProxyServer) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 47, 0, (void*)lpProxyServer, 0);
    }

    inline int GetProxySSL() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 48, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxySSL(int iProxySSL) {
      void* val = (void*)IPS64CAST(iProxySSL);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 48, 0, val, 0);
    }
    inline char* GetProxyUser() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 49, 0, 0);
      return (char*)val;
    }

    inline int SetProxyUser(const char *lpProxyUser) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 49, 0, (void*)lpProxyUser, 0);
    }

    inline char* GetReferer() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 50, 0, 0);
      return (char*)val;
    }

    inline int SetReferer(const char *lpReferer) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 50, 0, (void*)lpReferer, 0);
    }

    inline int GetResourceCount() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 51, 0, 0);
      return (int)(long)val;
    }

    inline char* GetResourceContentLanguage(int iResourceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 52, iResourceIndex, 0);
      return (char*)val;
    }


    inline char* GetResourceContentLength(int iResourceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 53, iResourceIndex, 0);
      return (char*)val;
    }


    inline char* GetResourceContentType(int iResourceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 54, iResourceIndex, 0);
      return (char*)val;
    }


    inline char* GetResourceDisplayName(int iResourceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 55, iResourceIndex, 0);
      return (char*)val;
    }


    inline char* GetResourceLastModified(int iResourceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 56, iResourceIndex, 0);
      return (char*)val;
    }


    inline char* GetResourceURI(int iResourceIndex) {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 57, iResourceIndex, 0);
      return (char*)val;
    }


    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 58, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 58, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 59, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 59, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 60, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 60, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 61, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 61, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 62, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 62, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 63, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 63, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 64, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline char* GetStatusLine() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 65, 0, 0);
      return (char*)val;
    }


    inline int GetTimeout() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 66, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 66, 0, val, 0);
    }
    inline int GetTransferredData(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 67, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }


    inline ns_int64 GetTransferredDataLimit() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_WebDAVS_Get(m_pObj, 68, 0, 0);
      return *pval;
    }

    inline int SetTransferredDataLimit(ns_int64 lTransferredDataLimit) {
      void* val = (void*)(&lTransferredDataLimit);
      return IPWorksSSL_WebDAVS_Set(m_pObj, 68, 0, val, 0);
    }

    inline char* GetTransferredHeaders() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 69, 0, 0);
      return (char*)val;
    }


    inline char* GetUser() {
      void* val = IPWorksSSL_WebDAVS_Get(m_pObj, 70, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 70, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline int AddCookie(const char* lpszCookieName, const char* lpszCookieValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszCookieName), (void*)IPS64CAST(lpszCookieValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 2, 2, param, cbparam);
      
      
    }
    inline int AddNamespace(const char* lpszPrefix, const char* lpszNamespaceURI) {
      void *param[2+1] = {(void*)IPS64CAST(lpszPrefix), (void*)IPS64CAST(lpszNamespaceURI), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 3, 2, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_WebDAVS_Do(m_pObj, 4, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int CopyResource(const char* lpszSourceResourceURI, const char* lpszDestinationResourceURI) {
      void *param[2+1] = {(void*)IPS64CAST(lpszSourceResourceURI), (void*)IPS64CAST(lpszDestinationResourceURI), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 5, 2, param, cbparam);
      
      
    }
    inline int DeleteResource(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 6, 1, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int FindProperties(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 8, 1, param, cbparam);
      
      
    }
    inline int FindPropertyNames(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 9, 1, param, cbparam);
      
      
    }
    inline char* GetProperty(const char* lpszPropertyName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszPropertyName), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_WebDAVS_Do(m_pObj, 10, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int GetResource(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 11, 1, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 12, 0, param, cbparam);
      
      
    }
    inline int ListDirectory(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 13, 1, param, cbparam);
      
      
    }
    inline int LockResource(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 14, 1, param, cbparam);
      
      
    }
    inline int MakeDirectory(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 15, 1, param, cbparam);
      
      
    }
    inline int MoveResource(const char* lpszSourceResourceURI, const char* lpszDestinationResourceURI) {
      void *param[2+1] = {(void*)IPS64CAST(lpszSourceResourceURI), (void*)IPS64CAST(lpszDestinationResourceURI), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 16, 2, param, cbparam);
      
      
    }
    inline int PatchProperties(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 17, 1, param, cbparam);
      
      
    }
    inline int PostToResource(const char* lpszResourceURI, const char* lpPostData, int lenPostData) {
      void *param[2+1] = {(void*)IPS64CAST(lpszResourceURI), (void*)IPS64CAST(lpPostData), 0};
      int cbparam[2+1] = {0, lenPostData, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 18, 2, param, cbparam);
      
      
    }
    inline int PutResource(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 19, 1, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 20, 0, param, cbparam);
      
      
    }
    inline int SetProperty(const char* lpszPropertyName, const char* lpszPropertyValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszPropertyName), (void*)IPS64CAST(lpszPropertyValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 22, 2, param, cbparam);
      
      
    }
    inline int UnLockResource(const char* lpszResourceURI) {
      void *param[1+1] = {(void*)IPS64CAST(lpszResourceURI), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 24, 1, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebDAVSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebDAVSConnectionStatusEventParamsW;

typedef struct {
  LPWSTR ResourceURI;
  LPWSTR DisplayName;
  LPWSTR ContentLanguage;
  LPWSTR ContentLength;
  LPWSTR ContentType;
  LPWSTR LastModified;
  int reserved;
} WebDAVSDirListEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebDAVSDisconnectedEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} WebDAVSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} WebDAVSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} WebDAVSHeaderEventParamsW;

typedef struct {
  LPWSTR Location;
  int Accept;
  int reserved;
} WebDAVSRedirectEventParamsW;

typedef struct {
  LPWSTR ResourceURI;
  LPWSTR ResourceProperties;
  int reserved;
} WebDAVSResourcePropertiesEventParamsW;

typedef struct {
  LPWSTR Name;
  LPWSTR Value;
  LPWSTR Expires;
  LPWSTR Domain;
  LPWSTR Path;
  int Secure;
  int reserved;
} WebDAVSSetCookieEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} WebDAVSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} WebDAVSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} WebDAVSStartTransferEventParamsW;

typedef struct {
  LPWSTR HTTPVersion;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebDAVSStatusEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int lenText;
  int reserved;
} WebDAVSTransferEventParamsW;



class WebDAVSW : public WebDAVS {

  public: //properties
  
    inline LPWSTR GetAccept() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAccept(LPWSTR lpAccept) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+1, 0, (void*)lpAccept, 0);
    }

    inline LPWSTR GetAuthorization() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+2, 0, 0);
    }

    inline int SetAuthorization(LPWSTR lpAuthorization) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+2, 0, (void*)lpAuthorization, 0);
    }







    inline LPWSTR GetCookieDomain(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+6, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieExpiration(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+7, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieName(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+8, iCookieIndex, 0);
    }

    inline int SetCookieName(int iCookieIndex, LPWSTR lpCookieName) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+8, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline LPWSTR GetCookiePath(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+9, iCookieIndex, 0);
    }





    inline LPWSTR GetCookieValue(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+11, iCookieIndex, 0);
    }

    inline int SetCookieValue(int iCookieIndex, LPWSTR lpCookieValue) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+11, iCookieIndex, (void*)lpCookieValue, 0);
    }







    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+15, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+16, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+18, 0, (void*)lpFirewallUser, 0);
    }





    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+10000+21, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+10000+21, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+22, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetLockOwner() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+23, 0, 0);
    }

    inline int SetLockOwner(LPWSTR lpLockOwner) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+23, 0, (void*)lpLockOwner, 0);
    }

    inline LPWSTR GetLockScope() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetLockScope(LPWSTR lpLockScope) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+24, 0, (void*)lpLockScope, 0);
    }



    inline LPWSTR GetLockTokens() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+26, 0, 0);
    }

    inline int SetLockTokens(LPWSTR lpLockTokens) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+26, 0, (void*)lpLockTokens, 0);
    }

    inline LPWSTR GetLockType() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+27, 0, 0);
    }

    inline int SetLockType(LPWSTR lpLockType) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+27, 0, (void*)lpLockType, 0);
    }



    inline LPWSTR GetNamespacePrefix(int iNamespaceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+29, iNamespaceIndex, 0);
    }

    inline int SetNamespacePrefix(int iNamespaceIndex, LPWSTR lpNamespacePrefix) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+29, iNamespaceIndex, (void*)lpNamespacePrefix, 0);
    }

    inline LPWSTR GetNamespaceURI(int iNamespaceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+30, iNamespaceIndex, 0);
    }

    inline int SetNamespaceURI(int iNamespaceIndex, LPWSTR lpNamespaceURI) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+30, iNamespaceIndex, (void*)lpNamespaceURI, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+31, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+31, 0, (void*)lpOtherHeaders, 0);
    }



    inline LPWSTR GetParsedHeaderField(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+33, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetParsedHeaderValue(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+34, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+35, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+35, 0, (void*)lpPassword, 0);
    }



    inline LPWSTR GetPropertyAttr(int iPropertyIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+37, iPropertyIndex, 0);
    }

    inline int SetPropertyAttr(int iPropertyIndex, LPWSTR lpPropertyAttr) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+37, iPropertyIndex, (void*)lpPropertyAttr, 0);
    }

    inline LPWSTR GetPropertyName(int iPropertyIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+38, iPropertyIndex, 0);
    }

    inline int SetPropertyName(int iPropertyIndex, LPWSTR lpPropertyName) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+38, iPropertyIndex, (void*)lpPropertyName, 0);
    }

    inline LPWSTR GetPropertyNamespaceURI(int iPropertyIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+39, iPropertyIndex, 0);
    }

    inline int SetPropertyNamespaceURI(int iPropertyIndex, LPWSTR lpPropertyNamespaceURI) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+39, iPropertyIndex, (void*)lpPropertyNamespaceURI, 0);
    }



    inline LPWSTR GetPropertyStatus(int iPropertyIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+41, iPropertyIndex, 0);
    }



    inline LPWSTR GetPropertyValue(int iPropertyIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+42, iPropertyIndex, 0);
    }

    inline int SetPropertyValue(int iPropertyIndex, LPWSTR lpPropertyValue) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+42, iPropertyIndex, (void*)lpPropertyValue, 0);
    }





    inline LPWSTR GetProxyPassword() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+45, 0, 0);
    }

    inline int SetProxyPassword(LPWSTR lpProxyPassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+45, 0, (void*)lpProxyPassword, 0);
    }



    inline LPWSTR GetProxyServer() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+47, 0, 0);
    }

    inline int SetProxyServer(LPWSTR lpProxyServer) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+47, 0, (void*)lpProxyServer, 0);
    }



    inline LPWSTR GetProxyUser() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+49, 0, 0);
    }

    inline int SetProxyUser(LPWSTR lpProxyUser) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+49, 0, (void*)lpProxyUser, 0);
    }

    inline LPWSTR GetReferer() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+50, 0, 0);
    }

    inline int SetReferer(LPWSTR lpReferer) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+50, 0, (void*)lpReferer, 0);
    }



    inline LPWSTR GetResourceContentLanguage(int iResourceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+52, iResourceIndex, 0);
    }



    inline LPWSTR GetResourceContentLength(int iResourceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+53, iResourceIndex, 0);
    }



    inline LPWSTR GetResourceContentType(int iResourceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+54, iResourceIndex, 0);
    }



    inline LPWSTR GetResourceDisplayName(int iResourceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+55, iResourceIndex, 0);
    }



    inline LPWSTR GetResourceLastModified(int iResourceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+56, iResourceIndex, 0);
    }



    inline LPWSTR GetResourceURI(int iResourceIndex) {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+57, iResourceIndex, 0);
    }



    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+58, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+58, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 58, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 58, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+59, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+59, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 59, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 59, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+60, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+60, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 60, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 60, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+61, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+61, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+63, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+63, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+64, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 64, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }

    inline LPWSTR GetStatusLine() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+65, 0, 0);
    }





    inline LPWSTR GetTransferredData() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+67, 0, 0);
    }


    inline int GetTransferredDataB(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_WebDAVS_Get(m_pObj, 67, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }



    inline LPWSTR GetTransferredHeaders() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+69, 0, 0);
    }



    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_WebDAVS_Get(m_pObj, 10000+70, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_WebDAVS_Set(m_pObj, 10000+70, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnected(WebDAVSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(WebDAVSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDirList(WebDAVSDirListEventParamsW *e) {return 0;}
    virtual int FireDisconnected(WebDAVSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(WebDAVSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(WebDAVSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(WebDAVSHeaderEventParamsW *e) {return 0;}
    virtual int FireRedirect(WebDAVSRedirectEventParamsW *e) {return 0;}
    virtual int FireResourceProperties(WebDAVSResourcePropertiesEventParamsW *e) {return 0;}
    virtual int FireSetCookie(WebDAVSSetCookieEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(WebDAVSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(WebDAVSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(WebDAVSStartTransferEventParamsW *e) {return 0;}
    virtual int FireStatus(WebDAVSStatusEventParamsW *e) {return 0;}
    virtual int FireTransfer(WebDAVSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int WebDAVSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            WebDAVSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            WebDAVSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 3: {
            WebDAVSDirListEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]),  0};
            ret_code = FireDirList(&e);
            break;
         }
         case 4: {
            WebDAVSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 5: {
            WebDAVSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 6: {
            WebDAVSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 7: {
            WebDAVSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 8: {
            WebDAVSRedirectEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = FireRedirect(&e);
            param[1] = (void*)(e.Accept);
            break;
         }
         case 9: {
            WebDAVSResourcePropertiesEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireResourceProperties(&e);
            break;
         }
         case 10: {
            WebDAVSSetCookieEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = FireSetCookie(&e);
            break;
         }
         case 11: {
            WebDAVSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 12: {
            WebDAVSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 13: {
            WebDAVSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 14: {
            WebDAVSStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireStatus(&e);
            break;
         }
         case 15: {
            WebDAVSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(WebDAVSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(WebDAVSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDirList(WebDAVSDirListEventParams *e) {return -10000;}
    virtual int FireDisconnected(WebDAVSDisconnectedEventParams *e) {return -10000;}
    virtual int FireEndTransfer(WebDAVSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(WebDAVSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(WebDAVSHeaderEventParams *e) {return -10000;}
    virtual int FireRedirect(WebDAVSRedirectEventParams *e) {return -10000;}
    virtual int FireResourceProperties(WebDAVSResourcePropertiesEventParams *e) {return -10000;}
    virtual int FireSetCookie(WebDAVSSetCookieEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(WebDAVSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(WebDAVSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(WebDAVSStartTransferEventParams *e) {return -10000;}
    virtual int FireStatus(WebDAVSStatusEventParams *e) {return -10000;}
    virtual int FireTransfer(WebDAVSTransferEventParams *e) {return -10000;}

  public: //methods

    inline int AddCookie(LPWSTR lpszCookieName, LPWSTR lpszCookieValue) {
      void *param[2+1] = {(void*)lpszCookieName, (void*)lpszCookieValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+2, 2, param, cbparam);
      
    }
    inline int AddNamespace(LPWSTR lpszPrefix, LPWSTR lpszNamespaceURI) {
      void *param[2+1] = {(void*)lpszPrefix, (void*)lpszNamespaceURI, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+3, 2, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_WebDAVS_Do(m_pObj, 10000+4, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int CopyResource(LPWSTR lpszSourceResourceURI, LPWSTR lpszDestinationResourceURI) {
      void *param[2+1] = {(void*)lpszSourceResourceURI, (void*)lpszDestinationResourceURI, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+5, 2, param, cbparam);
      
    }
    inline int DeleteResource(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+6, 1, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int FindProperties(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+8, 1, param, cbparam);
      
    }
    inline int FindPropertyNames(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+9, 1, param, cbparam);
      
    }
    inline LPWSTR GetProperty(LPWSTR lpszPropertyName) {
      void *param[1+1] = {(void*)lpszPropertyName, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_WebDAVS_Do(m_pObj, 10000+10, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int GetResource(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+11, 1, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+12, 0, param, cbparam);
      
    }
    inline int ListDirectory(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+13, 1, param, cbparam);
      
    }
    inline int LockResource(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+14, 1, param, cbparam);
      
    }
    inline int MakeDirectory(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+15, 1, param, cbparam);
      
    }
    inline int MoveResource(LPWSTR lpszSourceResourceURI, LPWSTR lpszDestinationResourceURI) {
      void *param[2+1] = {(void*)lpszSourceResourceURI, (void*)lpszDestinationResourceURI, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+16, 2, param, cbparam);
      
    }
    inline int PatchProperties(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+17, 1, param, cbparam);
      
    }
    inline int PostToResource(LPWSTR lpszResourceURI, LPWSTR lpPostData, int lenPostData) {
      void *param[2+1] = {(void*)lpszResourceURI, (void*)lpPostData, 0};
      int cbparam[2+1] = {0, lenPostData, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+18, 2, param, cbparam);
      
    }
    inline int PutResource(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+19, 1, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+20, 0, param, cbparam);
      
    }
    inline int SetProperty(LPWSTR lpszPropertyName, LPWSTR lpszPropertyValue) {
      void *param[2+1] = {(void*)lpszPropertyName, (void*)lpszPropertyValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+22, 2, param, cbparam);
      
    }
    inline int UnLockResource(LPWSTR lpszResourceURI) {
      void *param[1+1] = {(void*)lpszResourceURI, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebDAVS_Do(m_pObj, 10000+24, 1, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_WEBDAVS_H_




