/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_XMPPS_H_
#define _IPWORKSSSL_XMPPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//XMPPBuddySubscriptions
#define ST_NONE                                            0
#define ST_TO                                              1
#define ST_FROM                                            2
#define ST_BOTH                                            3
#define ST_REMOVE                                          4

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TJabberMessageTypes
#define MT_NORMAL                                          0
#define MT_CHAT                                            1
#define MT_GROUP_CHAT                                      2
#define MT_HEADLINE                                        3
#define MT_ERROR                                           4

//TJabberPresenceCodes
#define PC_OFFLINE                                         0
#define PC_CHAT                                            1
#define PC_AWAY                                            2
#define PC_XA                                              3
#define PC_DND                                             4

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_XMPPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_XMPPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_XMPPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_XMPPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_XMPPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_XMPPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_XMPPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_XMPPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_XMPPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_XMPPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int BuddyIdx;
  int reserved;
} XMPPSBuddyUpdateEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} XMPPSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} XMPPSConnectionStatusEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} XMPPSDisconnectedEventParams;

typedef struct {
  int Direction;
  const char* FileId;
  const char* Filename;
  int Success;
  int reserved;
} XMPPSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} XMPPSErrorEventParams;

typedef struct {
  const char* Iq;
  const char* Id;
  const char* From;
  const char* IqType;
  int Ignore;
  int reserved;
} XMPPSIQEventParams;

typedef struct {
  const char* MessageId;
  const char* From;
  const char* Domain;
  const char* Resource;
  int MessageType;
  const char* Subject;
  const char* MessageThread;
  const char* MessageText;
  const char* MessageHTML;
  const char* Other;
  int reserved;
} XMPPSMessageInEventParams;

typedef struct {
  int Direction;
  const char* Pi;
  int reserved;
} XMPPSPITrailEventParams;

typedef struct {
  const char* User;
  const char* Domain;
  const char* Resource;
  int Availability;
  const char* Status;
  int reserved;
} XMPPSPresenceEventParams;

typedef struct {
  int reserved;
} XMPPSReadyToSendEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} XMPPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} XMPPSSSLStatusEventParams;

typedef struct {
  int Direction;
  const char* FileId;
  const char* User;
  const char* Domain;
  const char* Resource;
  const char* Filename;
  const char* Datetime;
  ns_int64 *pSize;
  int Accept;
  int reserved;
} XMPPSStartTransferEventParams;

typedef struct {
  const char* From;
  const char* Domain;
  int Accept;
  int reserved;
} XMPPSSubscriptionRequestEventParams;

typedef struct {
  int reserved;
} XMPPSSyncEventParams;

typedef struct {
  int Direction;
  const char* FileId;
  const char* Filename;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int Cancel;
  int lenText;
  int reserved;
} XMPPSTransferEventParams;



class XMPPS {
  
  public: //events
  
    virtual int FireBuddyUpdate(XMPPSBuddyUpdateEventParams *e) {return 0;}
    virtual int FireConnected(XMPPSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(XMPPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDisconnected(XMPPSDisconnectedEventParams *e) {return 0;}
    virtual int FireEndTransfer(XMPPSEndTransferEventParams *e) {return 0;}
    virtual int FireError(XMPPSErrorEventParams *e) {return 0;}
    virtual int FireIQ(XMPPSIQEventParams *e) {return 0;}
    virtual int FireMessageIn(XMPPSMessageInEventParams *e) {return 0;}
    virtual int FirePITrail(XMPPSPITrailEventParams *e) {return 0;}
    virtual int FirePresence(XMPPSPresenceEventParams *e) {return 0;}
    virtual int FireReadyToSend(XMPPSReadyToSendEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(XMPPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(XMPPSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(XMPPSStartTransferEventParams *e) {return 0;}
    virtual int FireSubscriptionRequest(XMPPSSubscriptionRequestEventParams *e) {return 0;}
    virtual int FireSync(XMPPSSyncEventParams *e) {return 0;}
    virtual int FireTransfer(XMPPSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL XMPPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((XMPPS*)lpObj)->XMPPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            XMPPSBuddyUpdateEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((XMPPS*)lpObj)->FireBuddyUpdate(&e);
            break;
         }
         case 2: {
            XMPPSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((XMPPS*)lpObj)->FireConnected(&e);
            break;
         }
         case 3: {
            XMPPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((XMPPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 4: {
            XMPPSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((XMPPS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 5: {
            XMPPSEndTransferEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (int)IPS64CAST(param[3]),  0};
            ret_code = ((XMPPS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 6: {
            XMPPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((XMPPS*)lpObj)->FireError(&e);
            break;
         }
         case 7: {
            XMPPSIQEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]),  0};
            ret_code = ((XMPPS*)lpObj)->FireIQ(&e);
            param[4] = (void*)IPS64CAST(e.Ignore);
            break;
         }
         case 8: {
            XMPPSMessageInEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (char*)IPS64CAST(param[6]), (char*)IPS64CAST(param[7]), (char*)IPS64CAST(param[8]), (char*)IPS64CAST(param[9]),  0};
            ret_code = ((XMPPS*)lpObj)->FireMessageIn(&e);
            break;
         }
         case 9: {
            XMPPSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((XMPPS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 10: {
            XMPPSPresenceEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (int)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]),  0};
            ret_code = ((XMPPS*)lpObj)->FirePresence(&e);
            break;
         }
         case 11: {
            XMPPSReadyToSendEventParams e = { 0};
            ret_code = ((XMPPS*)lpObj)->FireReadyToSend(&e);
            break;
         }
         case 12: {
            XMPPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((XMPPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 13: {
            XMPPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((XMPPS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 14: {
            XMPPSStartTransferEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (char*)IPS64CAST(param[6]), (ns_int64*)IPS64CAST(param[7]), (int)IPS64CAST(param[8]),  0};
            ret_code = ((XMPPS*)lpObj)->FireStartTransfer(&e);
            param[5] = (void*)IPS64CAST(e.Filename);
            param[8] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 15: {
            XMPPSSubscriptionRequestEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = ((XMPPS*)lpObj)->FireSubscriptionRequest(&e);
            param[2] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 16: {
            XMPPSSyncEventParams e = { 0};
            ret_code = ((XMPPS*)lpObj)->FireSync(&e);
            break;
         }
         case 17: {
            XMPPSTransferEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (ns_int64*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (int)IPS64CAST(cbparam[5]),  0};
            ret_code = ((XMPPS*)lpObj)->FireTransfer(&e);
            param[6] = (void*)IPS64CAST(e.Cancel);
            break;
         }

      }
      return ret_code;
    }

    virtual int XMPPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    XMPPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_70) {
      m_pObj = IPWorksSSL_XMPPS_Create(XMPPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~XMPPS() {
      IPWorksSSL_XMPPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_XMPPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_XMPPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_XMPPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAuthDomain() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAuthDomain(const char *lpAuthDomain) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 1, 0, (void*)lpAuthDomain, 0);
    }

    inline char* GetAuthMethods() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }

    inline int SetAuthMethods(const char *lpAuthMethods) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 2, 0, (void*)lpAuthMethods, 0);
    }

    inline int GetBuddyCount() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }

    inline char* GetBuddyGroup(int iBuddyIndex) {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 4, iBuddyIndex, 0);
      return (char*)val;
    }

    inline int SetBuddyGroup(int iBuddyIndex, const char *lpBuddyGroup) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 4, iBuddyIndex, (void*)lpBuddyGroup, 0);
    }

    inline char* GetBuddyId(int iBuddyIndex) {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 5, iBuddyIndex, 0);
      return (char*)val;
    }


    inline char* GetBuddyNickName(int iBuddyIndex) {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 6, iBuddyIndex, 0);
      return (char*)val;
    }

    inline int SetBuddyNickName(int iBuddyIndex, const char *lpBuddyNickName) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 6, iBuddyIndex, (void*)lpBuddyNickName, 0);
    }

    inline char* GetBuddyRealName(int iBuddyIndex) {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 7, iBuddyIndex, 0);
      return (char*)val;
    }


    inline int GetBuddySubscription(int iBuddyIndex) {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 8, iBuddyIndex, 0);
      return (int)(long)val;
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 9, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_XMPPS_Set(m_pObj, 9, 0, val, 0);
    }
    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_XMPPS_Set(m_pObj, 10, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 11, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_XMPPS_Set(m_pObj, 11, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 12, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 13, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 14, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_XMPPS_Set(m_pObj, 14, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 15, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIMPort() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 16, 0, 0);
      return (int)(long)val;
    }
    inline int SetIMPort(int iIMPort) {
      void* val = (void*)IPS64CAST(iIMPort);
      return IPWorksSSL_XMPPS_Set(m_pObj, 16, 0, val, 0);
    }
    inline char* GetIMServer() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }

    inline int SetIMServer(const char *lpIMServer) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 17, 0, (void*)lpIMServer, 0);
    }

    inline char* GetLocalDirectory() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetLocalDirectory(const char *lpLocalDirectory) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 18, 0, (void*)lpLocalDirectory, 0);
    }

    inline char* GetLocalFile() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 19, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 20, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetMessageHTML() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetMessageHTML(const char *lpMessageHTML) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 21, 0, (void*)lpMessageHTML, 0);
    }

    inline char* GetMessageOtherData() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetMessageOtherData(const char *lpMessageOtherData) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 22, 0, (void*)lpMessageOtherData, 0);
    }

    inline char* GetMessageSubject() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 23, 0, 0);
      return (char*)val;
    }

    inline int SetMessageSubject(const char *lpMessageSubject) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 23, 0, (void*)lpMessageSubject, 0);
    }

    inline char* GetMessageText() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }

    inline int SetMessageText(const char *lpMessageText) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 24, 0, (void*)lpMessageText, 0);
    }

    inline char* GetMessageThread() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }

    inline int SetMessageThread(const char *lpMessageThread) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 25, 0, (void*)lpMessageThread, 0);
    }

    inline int GetMessageType() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 26, 0, 0);
      return (int)(long)val;
    }
    inline int SetMessageType(int iMessageType) {
      void* val = (void*)IPS64CAST(iMessageType);
      return IPWorksSSL_XMPPS_Set(m_pObj, 26, 0, val, 0);
    }
    inline char* GetPassword() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 27, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 27, 0, (void*)lpPassword, 0);
    }

    inline int GetPresence() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 28, 0, 0);
      return (int)(long)val;
    }
    inline int SetPresence(int iPresence) {
      void* val = (void*)IPS64CAST(iPresence);
      return IPWorksSSL_XMPPS_Set(m_pObj, 28, 0, val, 0);
    }
    inline char* GetResource() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 29, 0, 0);
      return (char*)val;
    }

    inline int SetResource(const char *lpResource) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 29, 0, (void*)lpResource, 0);
    }

    inline char* GetServerDomain() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 30, 0, 0);
      return (char*)val;
    }

    inline int SetServerDomain(const char *lpServerDomain) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 30, 0, (void*)lpServerDomain, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 31, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 31, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 32, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 32, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 33, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 33, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 34, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 34, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 35, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_XMPPS_Set(m_pObj, 35, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 36, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 36, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 37, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 38, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_XMPPS_Set(m_pObj, 38, 0, val, 0);
    }
    inline char* GetStatus() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 39, 0, 0);
      return (char*)val;
    }

    inline int SetStatus(const char *lpStatus) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 39, 0, (void*)lpStatus, 0);
    }

    inline int GetTimeout() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 40, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_XMPPS_Set(m_pObj, 40, 0, val, 0);
    }
    inline char* GetUser() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 41, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 41, 0, (void*)lpUser, 0);
    }

    inline char* GetUserDomain() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 42, 0, 0);
      return (char*)val;
    }

    inline int SetUserDomain(const char *lpUserDomain) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 42, 0, (void*)lpUserDomain, 0);
    }

    inline int GetUserInfoCount() {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 43, 0, 0);
      return (int)(long)val;
    }
    inline int SetUserInfoCount(int iUserInfoCount) {
      void* val = (void*)IPS64CAST(iUserInfoCount);
      return IPWorksSSL_XMPPS_Set(m_pObj, 43, 0, val, 0);
    }
    inline char* GetUserInfoField(int iFieldIndex) {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 44, iFieldIndex, 0);
      return (char*)val;
    }

    inline int SetUserInfoField(int iFieldIndex, const char *lpUserInfoField) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 44, iFieldIndex, (void*)lpUserInfoField, 0);
    }

    inline char* GetUserInfoValue(int iFieldIndex) {
      void* val = IPWorksSSL_XMPPS_Get(m_pObj, 45, iFieldIndex, 0);
      return (char*)val;
    }

    inline int SetUserInfoValue(int iFieldIndex, const char *lpUserInfoValue) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 45, iFieldIndex, (void*)lpUserInfoValue, 0);
    }


  public: //methods

    inline int Add(const char* lpszJabberId, const char* lpszName, const char* lpszGroups) {
      void *param[3+1] = {(void*)IPS64CAST(lpszJabberId), (void*)IPS64CAST(lpszName), (void*)IPS64CAST(lpszGroups), 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 2, 3, param, cbparam);
      
      
    }
    inline int Cancel(const char* lpszJabberId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszJabberId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 3, 1, param, cbparam);
      
      
    }
    inline int ChangePassword(const char* lpszPassword) {
      void *param[1+1] = {(void*)IPS64CAST(lpszPassword), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 4, 1, param, cbparam);
      
      
    }
    inline int ChangePresence(int iPresenceCode, const char* lpszStatus) {
      void *param[2+1] = {(void*)IPS64CAST(iPresenceCode), (void*)IPS64CAST(lpszStatus), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 5, 2, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_XMPPS_Do(m_pObj, 6, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect(const char* lpszUser, const char* lpszPassword) {
      void *param[2+1] = {(void*)IPS64CAST(lpszUser), (void*)IPS64CAST(lpszPassword), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 7, 2, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline int ProbePresence(const char* lpszJabberId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszJabberId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 11, 1, param, cbparam);
      
      
    }
    inline int QueryRegister(const char* lpszXMPPServer) {
      void *param[1+1] = {(void*)IPS64CAST(lpszXMPPServer), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 12, 1, param, cbparam);
      
      
    }
    inline int Register(const char* lpszXMPPServer) {
      void *param[1+1] = {(void*)IPS64CAST(lpszXMPPServer), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 13, 1, param, cbparam);
      
      
    }
    inline int Remove(const char* lpszJabberId, const char* lpszName, const char* lpszGroup) {
      void *param[3+1] = {(void*)IPS64CAST(lpszJabberId), (void*)IPS64CAST(lpszName), (void*)IPS64CAST(lpszGroup), 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 14, 3, param, cbparam);
      
      
    }
    inline int RetrieveRoster() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 15, 0, param, cbparam);
      
      
    }
    inline int SendCommand(const char* lpszCommand) {
      void *param[1+1] = {(void*)IPS64CAST(lpszCommand), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 16, 1, param, cbparam);
      
      
    }
    inline int SendFile(const char* lpszJabberId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszJabberId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 17, 1, param, cbparam);
      
      
    }
    inline char* SendMessage(const char* lpszJabberId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszJabberId), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_XMPPS_Do(m_pObj, 18, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int SetUserInfoField(const char* lpszField, const char* lpszValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszField), (void*)IPS64CAST(lpszValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 21, 2, param, cbparam);
      
      
    }
    inline int SubscribeTo(const char* lpszJabberId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszJabberId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 22, 1, param, cbparam);
      
      
    }
    inline int Unregister() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 23, 0, param, cbparam);
      
      
    }
    inline int UnsubscribeTo(const char* lpszJabberId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszJabberId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 24, 1, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int BuddyIdx;
  int reserved;
} XMPPSBuddyUpdateEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} XMPPSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} XMPPSConnectionStatusEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} XMPPSDisconnectedEventParamsW;

typedef struct {
  int Direction;
  LPWSTR FileId;
  LPWSTR Filename;
  int Success;
  int reserved;
} XMPPSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} XMPPSErrorEventParamsW;

typedef struct {
  LPWSTR Iq;
  LPWSTR Id;
  LPWSTR From;
  LPWSTR IqType;
  int Ignore;
  int reserved;
} XMPPSIQEventParamsW;

typedef struct {
  LPWSTR MessageId;
  LPWSTR From;
  LPWSTR Domain;
  LPWSTR Resource;
  int MessageType;
  LPWSTR Subject;
  LPWSTR MessageThread;
  LPWSTR MessageText;
  LPWSTR MessageHTML;
  LPWSTR Other;
  int reserved;
} XMPPSMessageInEventParamsW;

typedef struct {
  int Direction;
  LPWSTR Pi;
  int reserved;
} XMPPSPITrailEventParamsW;

typedef struct {
  LPWSTR User;
  LPWSTR Domain;
  LPWSTR Resource;
  int Availability;
  LPWSTR Status;
  int reserved;
} XMPPSPresenceEventParamsW;

typedef struct {
  int reserved;
} XMPPSReadyToSendEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} XMPPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} XMPPSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  LPWSTR FileId;
  LPWSTR User;
  LPWSTR Domain;
  LPWSTR Resource;
  LPWSTR Filename;
  LPWSTR Datetime;
  ns_int64 *pSize;
  int Accept;
  int reserved;
} XMPPSStartTransferEventParamsW;

typedef struct {
  LPWSTR From;
  LPWSTR Domain;
  int Accept;
  int reserved;
} XMPPSSubscriptionRequestEventParamsW;

typedef struct {
  int reserved;
} XMPPSSyncEventParamsW;

typedef struct {
  int Direction;
  LPWSTR FileId;
  LPWSTR Filename;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int Cancel;
  int lenText;
  int reserved;
} XMPPSTransferEventParamsW;



class XMPPSW : public XMPPS {

  public: //properties
  
    inline LPWSTR GetAuthDomain() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAuthDomain(LPWSTR lpAuthDomain) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+1, 0, (void*)lpAuthDomain, 0);
    }

    inline LPWSTR GetAuthMethods() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+2, 0, 0);
    }

    inline int SetAuthMethods(LPWSTR lpAuthMethods) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+2, 0, (void*)lpAuthMethods, 0);
    }



    inline LPWSTR GetBuddyGroup(int iBuddyIndex) {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+4, iBuddyIndex, 0);
    }

    inline int SetBuddyGroup(int iBuddyIndex, LPWSTR lpBuddyGroup) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+4, iBuddyIndex, (void*)lpBuddyGroup, 0);
    }

    inline LPWSTR GetBuddyId(int iBuddyIndex) {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+5, iBuddyIndex, 0);
    }



    inline LPWSTR GetBuddyNickName(int iBuddyIndex) {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+6, iBuddyIndex, 0);
    }

    inline int SetBuddyNickName(int iBuddyIndex, LPWSTR lpBuddyNickName) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+6, iBuddyIndex, (void*)lpBuddyNickName, 0);
    }

    inline LPWSTR GetBuddyRealName(int iBuddyIndex) {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+7, iBuddyIndex, 0);
    }











    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+12, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+12, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+13, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+13, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+15, 0, (void*)lpFirewallUser, 0);
    }



    inline LPWSTR GetIMServer() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetIMServer(LPWSTR lpIMServer) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+17, 0, (void*)lpIMServer, 0);
    }

    inline LPWSTR GetLocalDirectory() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetLocalDirectory(LPWSTR lpLocalDirectory) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+18, 0, (void*)lpLocalDirectory, 0);
    }

    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+19, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+20, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+20, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetMessageHTML() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetMessageHTML(LPWSTR lpMessageHTML) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+21, 0, (void*)lpMessageHTML, 0);
    }

    inline LPWSTR GetMessageOtherData() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetMessageOtherData(LPWSTR lpMessageOtherData) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+22, 0, (void*)lpMessageOtherData, 0);
    }

    inline LPWSTR GetMessageSubject() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+23, 0, 0);
    }

    inline int SetMessageSubject(LPWSTR lpMessageSubject) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+23, 0, (void*)lpMessageSubject, 0);
    }

    inline LPWSTR GetMessageText() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetMessageText(LPWSTR lpMessageText) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+24, 0, (void*)lpMessageText, 0);
    }

    inline LPWSTR GetMessageThread() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetMessageThread(LPWSTR lpMessageThread) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+25, 0, (void*)lpMessageThread, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+27, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+27, 0, (void*)lpPassword, 0);
    }



    inline LPWSTR GetResource() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+29, 0, 0);
    }

    inline int SetResource(LPWSTR lpResource) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+29, 0, (void*)lpResource, 0);
    }

    inline LPWSTR GetServerDomain() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+30, 0, 0);
    }

    inline int SetServerDomain(LPWSTR lpServerDomain) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+30, 0, (void*)lpServerDomain, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+31, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+31, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 31, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 31, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+32, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+32, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 32, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 32, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+33, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+33, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 33, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 33, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+34, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+34, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+36, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+36, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+37, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_XMPPS_Get(m_pObj, 37, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }



    inline LPWSTR GetStatus() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+39, 0, 0);
    }

    inline int SetStatus(LPWSTR lpStatus) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+39, 0, (void*)lpStatus, 0);
    }



    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+41, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+41, 0, (void*)lpUser, 0);
    }

    inline LPWSTR GetUserDomain() {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+42, 0, 0);
    }

    inline int SetUserDomain(LPWSTR lpUserDomain) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+42, 0, (void*)lpUserDomain, 0);
    }



    inline LPWSTR GetUserInfoField(int iFieldIndex) {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+44, iFieldIndex, 0);
    }

    inline int SetUserInfoField(int iFieldIndex, LPWSTR lpUserInfoField) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+44, iFieldIndex, (void*)lpUserInfoField, 0);
    }

    inline LPWSTR GetUserInfoValue(int iFieldIndex) {
      return (LPWSTR)IPWorksSSL_XMPPS_Get(m_pObj, 10000+45, iFieldIndex, 0);
    }

    inline int SetUserInfoValue(int iFieldIndex, LPWSTR lpUserInfoValue) {
      return IPWorksSSL_XMPPS_Set(m_pObj, 10000+45, iFieldIndex, (void*)lpUserInfoValue, 0);
    }



  public: //events
  
    virtual int FireBuddyUpdate(XMPPSBuddyUpdateEventParamsW *e) {return 0;}
    virtual int FireConnected(XMPPSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(XMPPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDisconnected(XMPPSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(XMPPSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(XMPPSErrorEventParamsW *e) {return 0;}
    virtual int FireIQ(XMPPSIQEventParamsW *e) {return 0;}
    virtual int FireMessageIn(XMPPSMessageInEventParamsW *e) {return 0;}
    virtual int FirePITrail(XMPPSPITrailEventParamsW *e) {return 0;}
    virtual int FirePresence(XMPPSPresenceEventParamsW *e) {return 0;}
    virtual int FireReadyToSend(XMPPSReadyToSendEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(XMPPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(XMPPSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(XMPPSStartTransferEventParamsW *e) {return 0;}
    virtual int FireSubscriptionRequest(XMPPSSubscriptionRequestEventParamsW *e) {return 0;}
    virtual int FireSync(XMPPSSyncEventParamsW *e) {return 0;}
    virtual int FireTransfer(XMPPSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int XMPPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            XMPPSBuddyUpdateEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireBuddyUpdate(&e);
            break;
         }
         case 2: {
            XMPPSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 3: {
            XMPPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 4: {
            XMPPSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 5: {
            XMPPSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (int)IPS64CAST(param[3]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 6: {
            XMPPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 7: {
            XMPPSIQEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]),  0};
            ret_code = FireIQ(&e);
            param[4] = (void*)(e.Ignore);
            break;
         }
         case 8: {
            XMPPSMessageInEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (LPWSTR)IPS64CAST(param[6]), (LPWSTR)IPS64CAST(param[7]), (LPWSTR)IPS64CAST(param[8]), (LPWSTR)IPS64CAST(param[9]),  0};
            ret_code = FireMessageIn(&e);
            break;
         }
         case 9: {
            XMPPSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 10: {
            XMPPSPresenceEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (int)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]),  0};
            ret_code = FirePresence(&e);
            break;
         }
         case 11: {
            XMPPSReadyToSendEventParamsW e = { 0};
            ret_code = FireReadyToSend(&e);
            break;
         }
         case 12: {
            XMPPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 13: {
            XMPPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 14: {
            XMPPSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (LPWSTR)IPS64CAST(param[6]), (ns_int64*)IPS64CAST(param[7]), (int)IPS64CAST(param[8]),  0};
            ret_code = FireStartTransfer(&e);
            param[5] = (void*)(e.Filename);
            param[8] = (void*)(e.Accept);
            break;
         }
         case 15: {
            XMPPSSubscriptionRequestEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = FireSubscriptionRequest(&e);
            param[2] = (void*)(e.Accept);
            break;
         }
         case 16: {
            XMPPSSyncEventParamsW e = { 0};
            ret_code = FireSync(&e);
            break;
         }
         case 17: {
            XMPPSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (ns_int64*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (int)IPS64CAST(param[6]), (int)IPS64CAST(cbparam[5]),  0};
            ret_code = FireTransfer(&e);
            param[6] = (void*)(e.Cancel);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireBuddyUpdate(XMPPSBuddyUpdateEventParams *e) {return -10000;}
    virtual int FireConnected(XMPPSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(XMPPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDisconnected(XMPPSDisconnectedEventParams *e) {return -10000;}
    virtual int FireEndTransfer(XMPPSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(XMPPSErrorEventParams *e) {return -10000;}
    virtual int FireIQ(XMPPSIQEventParams *e) {return -10000;}
    virtual int FireMessageIn(XMPPSMessageInEventParams *e) {return -10000;}
    virtual int FirePITrail(XMPPSPITrailEventParams *e) {return -10000;}
    virtual int FirePresence(XMPPSPresenceEventParams *e) {return -10000;}
    virtual int FireReadyToSend(XMPPSReadyToSendEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(XMPPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(XMPPSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(XMPPSStartTransferEventParams *e) {return -10000;}
    virtual int FireSubscriptionRequest(XMPPSSubscriptionRequestEventParams *e) {return -10000;}
    virtual int FireSync(XMPPSSyncEventParams *e) {return -10000;}
    virtual int FireTransfer(XMPPSTransferEventParams *e) {return -10000;}

  public: //methods

    inline int Add(LPWSTR lpszJabberId, LPWSTR lpszName, LPWSTR lpszGroups) {
      void *param[3+1] = {(void*)lpszJabberId, (void*)lpszName, (void*)lpszGroups, 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+2, 3, param, cbparam);
      
    }
    inline int Cancel(LPWSTR lpszJabberId) {
      void *param[1+1] = {(void*)lpszJabberId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+3, 1, param, cbparam);
      
    }
    inline int ChangePassword(LPWSTR lpszPassword) {
      void *param[1+1] = {(void*)lpszPassword, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+4, 1, param, cbparam);
      
    }
    inline int ChangePresence(int iPresenceCode, LPWSTR lpszStatus) {
      void *param[2+1] = {(void*)iPresenceCode, (void*)lpszStatus, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+5, 2, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_XMPPS_Do(m_pObj, 10000+6, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect(LPWSTR lpszUser, LPWSTR lpszPassword) {
      void *param[2+1] = {(void*)lpszUser, (void*)lpszPassword, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+7, 2, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline int ProbePresence(LPWSTR lpszJabberId) {
      void *param[1+1] = {(void*)lpszJabberId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+11, 1, param, cbparam);
      
    }
    inline int QueryRegister(LPWSTR lpszXMPPServer) {
      void *param[1+1] = {(void*)lpszXMPPServer, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+12, 1, param, cbparam);
      
    }
    inline int Register(LPWSTR lpszXMPPServer) {
      void *param[1+1] = {(void*)lpszXMPPServer, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+13, 1, param, cbparam);
      
    }
    inline int Remove(LPWSTR lpszJabberId, LPWSTR lpszName, LPWSTR lpszGroup) {
      void *param[3+1] = {(void*)lpszJabberId, (void*)lpszName, (void*)lpszGroup, 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+14, 3, param, cbparam);
      
    }
    inline int RetrieveRoster() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+15, 0, param, cbparam);
      
    }
    inline int SendCommand(LPWSTR lpszCommand) {
      void *param[1+1] = {(void*)lpszCommand, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+16, 1, param, cbparam);
      
    }
    inline int SendFile(LPWSTR lpszJabberId) {
      void *param[1+1] = {(void*)lpszJabberId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+17, 1, param, cbparam);
      
    }
    inline LPWSTR SendMessage(LPWSTR lpszJabberId) {
      void *param[1+1] = {(void*)lpszJabberId, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_XMPPS_Do(m_pObj, 10000+18, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int SetUserInfoField(LPWSTR lpszField, LPWSTR lpszValue) {
      void *param[2+1] = {(void*)lpszField, (void*)lpszValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+21, 2, param, cbparam);
      
    }
    inline int SubscribeTo(LPWSTR lpszJabberId) {
      void *param[1+1] = {(void*)lpszJabberId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+22, 1, param, cbparam);
      
    }
    inline int Unregister() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+23, 0, param, cbparam);
      
    }
    inline int UnsubscribeTo(LPWSTR lpszJabberId) {
      void *param[1+1] = {(void*)lpszJabberId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_XMPPS_Do(m_pObj, 10000+24, 1, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_XMPPS_H_




